@@ -16,13 +16,6 @@
  * specific language governing permissions and limitations
  * under the License.
  */
-/**
- * deployment_admin.c
- *
- *  \date       Nov 7, 2011
- *  \author    	<a href="mailto:dev@celix.apache.org">Apache Celix Project Team</a>
- *  \copyright	Apache License, Version 2.0
- */
 
 #include <stddef.h>
 #include <stdlib.h>
@@ -37,10 +30,11 @@
 
 #include <curl/curl.h>
 #include <curl/easy.h>
-
+#include <stdbool.h>
 #include <uuid/uuid.h>
+#include <celix_bundle_context.h>
+
 
-#include "celixbool.h"
 #include "deployment_admin.h"
 #include "celix_errno.h"
 #include "bundle_context.h"
@@ -136,16 +130,6 @@ celix_status_t deploymentAdmin_create(bundle_context_pt context, deployment_admi
 			(*admin)->pollUrl = strdup(pollUrl);
 			(*admin)->auditlogUrl = strdup(auditlogUrl);
 
-//				log_store_pt store = NULL;
-//				log_t *log = NULL;
-//				log_sync_pt sync = NULL;
-//				logStore_create(subpool, &store);
-//				log_create(subpool, store, &log);
-//				logSync_create(subpool, (*admin)->targetIdentification, store, &sync);
-//
-//				log_log(log, 20000, NULL);
-
-
 			celixThread_create(&(*admin)->poller, NULL, deploymentAdmin_poll, *admin);
 		}
 	}
@@ -194,25 +178,29 @@ static celix_status_t deploymentAdmin_performRequest(deployment_admin_pt admin,
     curl = curl_easy_init();
 
     if (!curl) {
-        status = CELIX_BUNDLE_EXCEPTION;
-
         fw_log(celix_frameworkLogger_globalLogger(), CELIX_LOG_LEVEL_ERROR, "Error initializing curl.");
+        return CELIX_BUNDLE_EXCEPTION;
     }
 
-    char url[strlen(admin->auditlogUrl)+6];
-    sprintf(url, "%s/send", admin->auditlogUrl);
+    char* url;
+    int rc = asprintf(&url, "%s/send", admin->auditlogUrl);
+    status = rc < 0 ? CELIX_ENOMEM : CELIX_SUCCESS;
 
     if (status == CELIX_SUCCESS) {
             curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1);
             curl_easy_setopt(curl, CURLOPT_URL, url);
             curl_easy_setopt(curl, CURLOPT_POSTFIELDS, entry);
             res = curl_easy_perform(curl);
-
             if (res != CURLE_OK ) {
                 status = CELIX_BUNDLE_EXCEPTION;
-                fw_log(celix_frameworkLogger_globalLogger(), CELIX_LOG_LEVEL_ERROR, "Error sending auditlog, got curl error code %d", res);
+                fw_log(celix_frameworkLogger_globalLogger(), CELIX_LOG_LEVEL_ERROR, "Error sending auditlog to %s, got curl error code %d", url, res);
             }
+            free(url);
+    } else {
+        fw_log(celix_frameworkLogger_globalLogger(), CELIX_LOG_LEVEL_ERROR, "Error creating send url for audit log url: %s", admin->auditlogUrl);
     }
+    curl_easy_cleanup(curl);
+
 
     return status;
 }
@@ -594,10 +582,10 @@ celix_status_t deploymentAdmin_updateDeploymentPackageBundles(deployment_admin_p
 		deploymentPackage_getBundle(source, info->symbolicName, &updateBundle);
 		if (updateBundle != NULL) {
 			//printf("Update bundle from: %s\n", bundlePath);
-			bundle_update(updateBundle, bundlePath);
+            celix_bundleContext_updateBundle(admin->context, celix_bundle_getId(updateBundle), bundlePath);
 		} else {
 			//printf("Install bundle from: %s\n", bundlePath);
-			bundleContext_installBundle2(admin->context, bsn, bundlePath, &updateBundle);
+            celix_bundleContext_installBundle(admin->context, bundlePath, false);
 		}
 
         free(entry);
@@ -687,7 +675,7 @@ celix_status_t deploymentAdmin_processDeploymentPackageResources(deployment_admi
 					bundle_getEntry(bundle, "/", &entry);
 					deploymentPackage_getName(source, &name);
 
-					int length = strlen(entry) + strlen(name) + strlen(info->path) + 7;
+					length = strlen(entry) + strlen(name) + strlen(info->path) + 7;
 					char resourcePath[length];
 					snprintf(resourcePath, length, "%srepo/%s/%s", entry, name, info->path);
 					deploymentPackage_getName(source, &packageName);
