@@ -45,6 +45,7 @@
 #include "hash_map.h"
 #include "utils.h"
 #include "pubsub_tcp_handler.h"
+#include "celix_compiler.h"
 
 #define MAX_EVENTS   64
 #define MAX_DEFAULT_BUFFER_SIZE 4u
@@ -337,7 +338,7 @@ pubsub_tcpHandler_createEntry(pubsub_tcpHandler_t *handle, int fd, char *url, ch
         entry->connected = false;
         unsigned minimalMsgSize = entry->writeHeaderBufferSize + entry->writeFooterBufferSize;
         if ((minimalMsgSize > handle->maxMsgSize) && (handle->maxMsgSize)) {
-            L_ERROR("[TCP Socket] maxMsgSize (%d) < headerSize + FooterSize (%d): %s\n", handle->maxMsgSize, minimalMsgSize);
+            L_ERROR("[TCP Socket] maxMsgSize (%d) < headerSize + FooterSize (%d)\n", handle->maxMsgSize, minimalMsgSize);
         } else {
             entry->maxMsgSize = (handle->maxMsgSize) ? handle->maxMsgSize : LONG_MAX;
         }
@@ -378,7 +379,7 @@ pubsub_tcpHandler_freeEntry(psa_tcp_connection_entry_t *entry) {
 // Releases the Buffer
 //
 static inline void
-pubsub_tcpHandler_releaseEntryBuffer(pubsub_tcpHandler_t *handle, int fd, unsigned int index __attribute__((unused))) {
+pubsub_tcpHandler_releaseEntryBuffer(pubsub_tcpHandler_t *handle, int fd, unsigned int index CELIX_UNUSED) {
     psa_tcp_connection_entry_t *entry = hashMap_get(handle->connection_fd_map, (void *) (intptr_t) fd);
     if (entry != NULL) {
         entry->buffer = NULL;
@@ -553,62 +554,65 @@ int pubsub_tcpHandler_listen(pubsub_tcpHandler_t *handle, char *url) {
     psa_tcp_connection_entry_t *entry =
         hashMap_get(handle->connection_url_map, (void *) (intptr_t) url);
     celixThreadRwlock_unlock(&handle->dbLock);
-    if (entry == NULL) {
-        char protocol[] = "tcp";
-        int fd = pubsub_tcpHandler_open(handle, url);
-        rc = fd;
-        struct sockaddr_in *sin = pubsub_utils_url_from_fd(fd);
-        // Make handler fd entry
-        char *pUrl = pubsub_utils_url_get_url(sin, protocol);
+    if (entry != NULL) {
+        // already exist
+        return rc;
+    }
+    char protocol[] = "tcp";
+    int fd = pubsub_tcpHandler_open(handle, url);
+    rc = fd;
+    if (rc < 0) {
+        return rc;
+    }
+    struct sockaddr_in *sin = pubsub_utils_url_from_fd(fd);
+    // Make handler fd entry
+    char *pUrl = pubsub_utils_url_get_url(sin, protocol);
+    if (sin != NULL && pUrl != NULL) {
         entry = pubsub_tcpHandler_createEntry(handle, fd, pUrl, NULL, sin);
-        if (entry != NULL) {
-            __atomic_store_n(&entry->connected, true, __ATOMIC_RELEASE);
-            free(pUrl);
-            free(sin);
-            celixThreadRwlock_writeLock(&handle->dbLock);
-            if (rc >= 0) {
-                rc = listen(fd, SOMAXCONN);
-                if (rc != 0) {
-                    L_ERROR("[TCP Socket] Error listen: %s\n", strerror(errno));
-                    pubsub_tcpHandler_freeEntry(entry);
-                    entry = NULL;
-                }
+    }
+    free(pUrl);
+    free(sin);
+    if (entry != NULL) {
+        __atomic_store_n(&entry->connected, true, __ATOMIC_RELEASE);
+        if (rc >= 0) {
+            rc = listen(fd, SOMAXCONN);
+            if (rc != 0) {
+                L_ERROR("[TCP Socket] Error listen: %s\n", strerror(errno));
             }
-            if (rc >= 0) {
-                rc = pubsub_tcpHandler_makeNonBlocking(handle, fd);
-                if (rc < 0) {
-                    pubsub_tcpHandler_freeEntry(entry);
-                    entry = NULL;
-                }
-            }
-            if ((rc >= 0) && (handle->efd >= 0)) {
+        }
+        if (rc >= 0) {
+            rc = pubsub_tcpHandler_makeNonBlocking(handle, fd);
+        }
+        if ((rc >= 0) && (handle->efd >= 0)) {
 #if defined(__APPLE__)
-                struct kevent ev;
+            struct kevent ev;
                 EV_SET (&ev, fd, EVFILT_READ, EV_ADD | EV_ENABLE, 0, 0, 0);
                 rc = kevent(handle->efd, &ev, 1, NULL, 0, NULL);
 #else
-                struct epoll_event event;
-                bzero(&event, sizeof(event)); // zero the struct
-                event.events = EPOLLIN | EPOLLRDHUP | EPOLLERR;
-                event.data.fd = fd;
-                rc = epoll_ctl(handle->efd, EPOLL_CTL_ADD, fd, &event);
+            struct epoll_event event;
+            bzero(&event, sizeof(event)); // zero the struct
+            event.events = EPOLLIN | EPOLLRDHUP | EPOLLERR;
+            event.data.fd = fd;
+            rc = epoll_ctl(handle->efd, EPOLL_CTL_ADD, fd, &event);
 #endif
-                if (rc < 0) {
-                    L_ERROR("[TCP Socket] Cannot create poll: %s\n", strerror(errno));
-                    errno = 0;
-                    pubsub_tcpHandler_freeEntry(entry);
-                    entry = NULL;
-                }
-                if (entry) {
-                    L_INFO("[TCP Socket] Using %s for service annunciation", entry->url);
-                    hashMap_put(handle->interface_fd_map, (void *) (intptr_t) entry->fd, entry);
-                    hashMap_put(handle->interface_url_map, entry->url, entry);
-                }
+            if (rc == 0) {
+                L_INFO("[TCP Socket] Using %s for service annunciation", entry->url);
+                celixThreadRwlock_writeLock(&handle->dbLock);
+                hashMap_put(handle->interface_fd_map, (void *) (intptr_t) entry->fd, entry);
+                hashMap_put(handle->interface_url_map, entry->url, entry);
+                celixThreadRwlock_unlock(&handle->dbLock);
+                entry = NULL;
+            } else {
+                L_ERROR("[TCP Socket] Cannot create poll: %s\n", strerror(errno));
             }
-            celixThreadRwlock_unlock(&handle->dbLock);
-        } else {
-            L_ERROR("[TCP Socket] Error listen socket cannot bind to %s: %s\n", url ? url : "", strerror(errno));
         }
+        if (entry) {
+            pubsub_tcpHandler_freeEntry(entry);
+            entry = NULL;
+        }
+    } else {
+        L_ERROR("[TCP Socket] Error listen socket cannot bind to %s: %s\n", url ? url : "", strerror(errno));
+        close(fd);
     }
     return rc;
 }
@@ -1031,16 +1035,13 @@ int pubsub_tcpHandler_write(pubsub_tcpHandler_t *handle, pubsub_protocol_message
             message->header.payloadOffset = 0;
             message->header.isLastSegment = 1;
 
-            void *metadataData = NULL;
             size_t metadataSize = 0;
             if (message->metadata.metadata) {
-                metadataSize = entry->writeMetaBufferSize;
-                metadataData = entry->writeMetaBuffer;
+                handle->protocol->encodeMetadata(handle->protocol->handle, message, &entry->writeMetaBuffer, &entry->writeMetaBufferSize, &metadataSize);
                 // When maxMsgSize is smaller then meta data is disabled
-                if (metadataSize > entry->maxMsgSize) {
+               if (metadataSize > entry->maxMsgSize) {
                     metadataSize = 0;
                 }
-                handle->protocol->encodeMetadata(handle->protocol->handle, message, &metadataData, &metadataSize);
             }
 
             message->header.metadataSize = metadataSize;
@@ -1116,10 +1117,10 @@ int pubsub_tcpHandler_write(pubsub_tcpHandler_t *handle, pubsub_protocol_message
 
                 // Write optional metadata in vector buffer
                 if (allPayloadAdded &&
-                    (metadataSize != 0 && metadataData) &&
+                    (metadataSize != 0 && entry->writeMetaBuffer) &&
                     (msgPartSize < maxMsgSize) &&
                     (msg.msg_iovlen-1 < max_msg_iov_len)) {  // header is already included
-                    msg.msg_iov[msg.msg_iovlen].iov_base = metadataData;
+                    msg.msg_iov[msg.msg_iovlen].iov_base = entry->writeMetaBuffer;
                     msg.msg_iov[msg.msg_iovlen].iov_len = metadataSize;
                     msg.msg_iovlen++;
                     msgPartSize += metadataSize;
@@ -1192,16 +1193,13 @@ int pubsub_tcpHandler_write(pubsub_tcpHandler_t *handle, pubsub_protocol_message
                 } else if (msgPartSize) {
                     entry->retryCount = 0;
                     if (nbytes != msgPartSize) {
-                        L_ERROR("[TCP Socket] seq: %d MsgSize not correct: %d != %d (%s)\n", message->header.seqNr, msgPartSize, nbytes, strerror(errno));
+                        L_ERROR("[TCP Socket] seq: %d MsgSize not correct: %zu != %ld (%s)\n", message->header.seqNr, msgPartSize, nbytes, strerror(errno));
                     }
                 }
                 // Note: serialized Payload is deleted by serializer
                 if (payloadData && (payloadData != message->payload.payload)) {
                     free(payloadData);
                 }
-                if (metadataData && metadataSize > 0) {
-                    free(metadataData);
-                }
             }
             celixThreadMutex_unlock(&entry->writeMutex);
         }
@@ -1301,7 +1299,6 @@ int pubsub_tcpHandler_acceptHandler(pubsub_tcpHandler_t *handle, psa_tcp_connect
 #endif
         if (rc < 0) {
             pubsub_tcpHandler_freeEntry(entry);
-            free(entry);
             L_ERROR("[TCP Socket] Cannot create epoll\n");
         } else {
             // Call Accept Connection callback
