@@ -24,7 +24,6 @@
 #include <pubsub_constants.h>
 #include <pubsub/publisher.h>
 #include <utils.h>
-#include <zconf.h>
 #include <arpa/inet.h>
 #include <celix_log_helper.h>
 #include "pubsub_psa_tcp_constants.h"
@@ -32,6 +31,7 @@
 #include "pubsub_tcp_handler.h"
 #include "pubsub_tcp_common.h"
 #include <uuid/uuid.h>
+#include "celix_compiler.h"
 #include "celix_constants.h"
 #include <pubsub_utils.h>
 #include "pubsub_interceptors_handler.h"
@@ -95,8 +95,6 @@ static void *psa_tcp_getPublisherService(void *handle, const celix_bundle_t *req
 static void psa_tcp_ungetPublisherService(void *handle, const celix_bundle_t *requestingBundle,
                                           const celix_properties_t *svcProperties);
 
-static void delay_first_send_for_late_joiners(pubsub_tcp_topic_sender_t *sender);
-
 static int
 psa_tcp_topicPublicationSend(void *handle, unsigned int msgTypeId, const void *msg, celix_properties_t *metadata);
 
@@ -314,7 +312,7 @@ bool pubsub_tcpTopicSender_isPassive(pubsub_tcp_topic_sender_t *sender) {
 }
 
 static void *psa_tcp_getPublisherService(void *handle, const celix_bundle_t *requestingBundle,
-                                         const celix_properties_t *svcProperties __attribute__((unused))) {
+                                         const celix_properties_t *svcProperties CELIX_UNUSED) {
     pubsub_tcp_topic_sender_t *sender = handle;
     long bndId = celix_bundle_getId(requestingBundle);
 
@@ -338,7 +336,7 @@ static void *psa_tcp_getPublisherService(void *handle, const celix_bundle_t *req
 }
 
 static void psa_tcp_ungetPublisherService(void *handle, const celix_bundle_t *requestingBundle,
-                                          const celix_properties_t *svcProperties __attribute__((unused))) {
+                                          const celix_properties_t *svcProperties CELIX_UNUSED) {
     pubsub_tcp_topic_sender_t *sender = handle;
     long bndId = celix_bundle_getId(requestingBundle);
 
@@ -389,8 +387,6 @@ psa_tcp_topicPublicationSend(void *handle, unsigned int msgTypeId, const void *i
         return status;
     }
 
-    delay_first_send_for_late_joiners(sender);
-
     pubsub_protocol_message_t message;
     message.metadata.metadata = NULL;
     message.payload.payload = NULL;
@@ -432,19 +428,6 @@ psa_tcp_topicPublicationSend(void *handle, unsigned int msgTypeId, const void *i
     return status;
 }
 
-static void delay_first_send_for_late_joiners(pubsub_tcp_topic_sender_t *sender) {
-
-    static bool firstSend = true;
-
-    if (firstSend) {
-        if (sender->send_delay ) {
-            L_INFO("PSA_TCP_TP: Delaying first send for late joiners...\n");
-        }
-        usleep(sender->send_delay * 1000);
-        firstSend = false;
-    }
-}
-
 static int psa_tcp_localMsgTypeIdForMsgType(void *handle, const char *msgType, unsigned int *msgTypeId) {
     psa_tcp_bounded_service_entry_t* entry = handle;
     uint32_t msgId = pubsub_serializerHandler_getMsgId(entry->parent->serializerHandler, msgType);
