@@ -32,6 +32,7 @@
 #include "pubsub_psa_udpmc_constants.h"
 #include "pubsub_udpmc_topic_sender.h"
 #include "pubsub_udpmc_topic_receiver.h"
+#include "celix_compiler.h"
 
 
 #define L_DEBUG(...) \
@@ -324,7 +325,7 @@ celix_status_t pubsub_udpmcAdmin_setupTopicSender(void *handle, const char *scop
             hashMap_put(psa->topicSenders.map, key, sender);
         } else {
             free(key);
-            L_ERROR("[PSA UDPMC] Error creating a valid TopicSender. Endpoints are not valid %i", serializerSvcId);
+            L_ERROR("[PSA UDPMC] Error creating a valid TopicSender. Endpoints are not valid %ld", serializerSvcId);
         }
     } else {
         free(key);
@@ -537,7 +538,7 @@ celix_status_t pubsub_udpmcAdmin_removeEndpoint(void *handle, const celix_proper
     return status;
 }
 
-bool pubsub_udpmcAdmin_executeCommand(void *handle, const char *commandLine __attribute__((unused)), FILE *out, FILE *errStream __attribute__((unused))) {
+bool pubsub_udpmcAdmin_executeCommand(void *handle, const char *commandLine CELIX_UNUSED, FILE *out, FILE *errStream CELIX_UNUSED) {
     pubsub_udpmc_admin_t *psa = handle;
 
     fprintf(out, "\n");
@@ -605,7 +606,7 @@ void pubsub_udpmcAdmin_addSerializerSvc(void *handle, void *svc, const celix_pro
         L_INFO("[PSA_UDPMC] Ignoring serializer service without %s property", PUBSUB_SERIALIZER_TYPE_KEY);
         return;
     }
-    L_WARN("[PSA_UDPM] adding serializer %s %i", serType, svcId);
+    L_WARN("[PSA_UDPM] adding serializer %s %ld", serType, svcId);
 
     celixThreadMutex_lock(&psa->serializers.mutex);
     psa_udpmc_serializer_entry_t *entry = hashMap_get(psa->serializers.map, (void*)svcId);
