@@ -19,14 +19,17 @@
 
 #include <pubsub_serializer.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <pubsub/subscriber.h>
 #include <memory.h>
 #include <pubsub_constants.h>
 #include <assert.h>
 #include <pubsub_endpoint.h>
 #include <arpa/inet.h>
+#include "celix_compiler.h"
 #include <celix_log_helper.h>
 #include <math.h>
+#include <limits.h>
 #include "pubsub_websocket_topic_receiver.h"
 #include "pubsub_psa_websocket_constants.h"
 #include "pubsub_websocket_common.h"
@@ -54,11 +57,6 @@
 #define L_ERROR(...) \
     celix_logHelper_log(receiver->logHelper, CELIX_LOG_LEVEL_ERROR, __VA_ARGS__)
 
-typedef struct pubsub_websocket_rcv_buffer {
-    celix_thread_mutex_t mutex;
-    celix_array_list_t *list;     //List of received websocket messages (type: pubsub_websocket_msg_entry_t *)
-} pubsub_websocket_rcv_buffer_t;
-
 typedef struct pubsub_websocket_msg_entry {
     size_t msgSize;
     const char *msgData;
@@ -68,6 +66,8 @@ struct pubsub_websocket_topic_receiver {
     celix_bundle_context_t *ctx;
     celix_log_helper_t *logHelper;
     void *admin;
+    useconds_t timeoutUs;
+
     char *scope;
     char *topic;
     char scopeAndTopicFilter[5];
@@ -79,8 +79,6 @@ struct pubsub_websocket_topic_receiver {
     celix_websocket_service_t sockSvc;
     long svcId;
 
-    pubsub_websocket_rcv_buffer_t recvBuffer;
-
     struct {
         celix_thread_t thread;
         celix_thread_mutex_t mutex;
@@ -150,15 +148,24 @@ pubsub_websocket_topic_receiver_t* pubsub_websocketTopicReceiver_create(celix_bu
 
     receiver->uri = psa_websocket_createURI(scope, topic);
 
+    long timeoutMs = celix_bundleContext_getPropertyAsLong(ctx, PSA_WEBSOCKET_SUBSCRIBER_CONNECTION_TIMEOUT,
+                                                                PSA_WEBSOCKET_SUBSCRIBER_CONNECTION_DEFAULT_TIMEOUT);
+    receiver->timeoutUs = (useconds_t)((
+            (
+                (timeoutMs < PSA_WEBSOCKET_SUBSCRIBER_CONNECTION_MIN_TIMEOUT) 
+                || (timeoutMs > PSA_WEBSOCKET_SUBSCRIBER_CONNECTION_MAX_TIMEOUT)
+            )  // Protect against under and overflow
+            ? PSA_WEBSOCKET_SUBSCRIBER_CONNECTION_DEFAULT_TIMEOUT // Use default for excessive values
+            : timeoutMs)
+         * 1000);
+
     if (receiver->uri != NULL) {
         celixThreadMutex_create(&receiver->subscribers.mutex, NULL);
         celixThreadMutex_create(&receiver->requestedConnections.mutex, NULL);
         celixThreadMutex_create(&receiver->recvThread.mutex, NULL);
-        celixThreadMutex_create(&receiver->recvBuffer.mutex, NULL);
 
         receiver->subscribers.map = hashMap_create(NULL, NULL, NULL, NULL);
         receiver->requestedConnections.map = hashMap_create(utils_stringHash, NULL, utils_stringEquals, NULL);
-        arrayList_create(&receiver->recvBuffer.list);
     }
 
     //track subscribers
@@ -293,16 +300,6 @@ void pubsub_websocketTopicReceiver_destroy(pubsub_websocket_topic_receiver_t *re
         celixThreadMutex_destroy(&receiver->requestedConnections.mutex);
         celixThreadMutex_destroy(&receiver->recvThread.mutex);
 
-        celixThreadMutex_destroy(&receiver->recvBuffer.mutex);
-        int msgBufSize = celix_arrayList_size(receiver->recvBuffer.list);
-        while(msgBufSize > 0) {
-            pubsub_websocket_msg_entry_t *msg = celix_arrayList_get(receiver->recvBuffer.list, msgBufSize - 1);
-            free((void *) msg->msgData);
-            free(msg);
-            msgBufSize--;
-        }
-        celix_arrayList_destroy(receiver->recvBuffer.list);
-
         pubsubInterceptorsHandler_destroy(receiver->interceptorsHandler);
 
         free(receiver->uri);
@@ -344,7 +341,7 @@ void pubsub_websocketTopicReceiver_listConnections(pubsub_websocket_topic_receiv
 
 
 void pubsub_websocketTopicReceiver_connectTo(pubsub_websocket_topic_receiver_t *receiver, const char *socketAddress, long socketPort) {
-    L_DEBUG("[PSA_WEBSOCKET] TopicReceiver %s/%s ('%s') connecting to websocket address %s:li", receiver->scope == NULL ? "(null)" : receiver->scope, receiver->topic, receiver->uri, socketAddress, socketPort);
+    L_DEBUG("[PSA_WEBSOCKET] TopicReceiver %s/%s ('%s') connecting to websocket address %s:%ld", receiver->scope == NULL ? "(null)" : receiver->scope, receiver->topic, receiver->uri, socketAddress, socketPort);
 
     char *key = NULL;
     asprintf(&key, "%s:%li", socketAddress, socketPort);
@@ -421,7 +418,7 @@ static void pubsub_websocketTopicReceiver_addSubscriber(void *handle, void *svc,
     celixThreadMutex_unlock(&receiver->subscribers.mutex);
 }
 
-static void pubsub_websocketTopicReceiver_removeSubscriber(void *handle, void *svc __attribute__((unused)), const celix_properties_t *props) {
+static void pubsub_websocketTopicReceiver_removeSubscriber(void *handle, void *svc CELIX_UNUSED, const celix_properties_t *props) {
     pubsub_websocket_topic_receiver_t *receiver = handle;
 
     long svcId = celix_properties_getAsLong(props, OSGI_FRAMEWORK_SERVICE_ID, -1);
@@ -539,7 +536,7 @@ static void processMsg(pubsub_websocket_topic_receiver_t *receiver, const char *
             free(payload);
         } else {
             L_WARN("[PSA_WEBSOCKET_TR] Received unsupported message: "
-                   "ID = %s, major = %d, minor = %d, seqNr = %d, data valid? %s",
+                   "ID = %s, major = %"JSON_INTEGER_FORMAT", minor = %"JSON_INTEGER_FORMAT", seqNr = %"JSON_INTEGER_FORMAT", data valid? %s",
                    (jsId ? json_string_value(jsId) : "ERROR"),
                    json_integer_value(jsMajor), json_integer_value(jsMinor),
                    json_integer_value(jsSeqNr), (jsData ? "TRUE" : "FALSE"));
@@ -573,17 +570,8 @@ static void* psa_websocket_recvThread(void * data) {
         if (!allInitialized) {
             psa_websocket_initializeAllSubscribers(receiver);
         }
+        usleep(receiver->timeoutUs);
 
-        while(celix_arrayList_size(receiver->recvBuffer.list) > 0) {
-            celixThreadMutex_lock(&receiver->recvBuffer.mutex);
-            pubsub_websocket_msg_entry_t *msg = (pubsub_websocket_msg_entry_t *) celix_arrayList_get(receiver->recvBuffer.list, 0);
-            celix_arrayList_removeAt(receiver->recvBuffer.list, 0);
-            celixThreadMutex_unlock(&receiver->recvBuffer.mutex);
-
-            processMsg(receiver, msg->msgData, msg->msgSize);
-            free((void *)msg->msgData);
-            free(msg);
-        }
 
         celixThreadMutex_lock(&receiver->recvThread.mutex);
         running = receiver->recvThread.running;
@@ -617,7 +605,7 @@ static void psa_websocketTopicReceiver_ready(struct mg_connection *connection, v
                 entry = calloc(1, sizeof(*entry));
                 entry->key = key;
                 entry->uri = strndup(ri->request_uri, 1024 * 1024);
-                entry->socketAddress = strndup(ri->remote_addr, 1024 * 1024);
+                entry->socketAddress = strndup(ri->remote_addr, 48);
                 entry->socketPort = ri->remote_port;
                 entry->connected = true;
                 entry->statically = false;
@@ -634,23 +622,17 @@ static void psa_websocketTopicReceiver_ready(struct mg_connection *connection, v
 }
 
 
-static int psa_websocketTopicReceiver_data(struct mg_connection *connection __attribute__((unused)),
-                                            int op_code __attribute__((unused)),
+static int psa_websocketTopicReceiver_data(struct mg_connection *connection CELIX_UNUSED,
+                                            int op_code CELIX_UNUSED,
                                             char *data,
                                             size_t length,
                                             void *handle) {
+    
+
     //Received a websocket message, append this message to the buffer of the receiver.
     if (handle != NULL) {
         pubsub_websocket_topic_receiver_t *receiver = (pubsub_websocket_topic_receiver_t *) handle;
-
-        celixThreadMutex_lock(&receiver->recvBuffer.mutex);
-        pubsub_websocket_msg_entry_t *msg = malloc(sizeof(*msg));
-        const char *rcvdMsgData = malloc(length);
-        memcpy((void *) rcvdMsgData, data, length);
-        msg->msgData = rcvdMsgData;
-        msg->msgSize = length;
-        celix_arrayList_add(receiver->recvBuffer.list, msg);
-        celixThreadMutex_unlock(&receiver->recvBuffer.mutex);
+        processMsg(receiver, data, length);
     }
 
     return 1; //keep open (non-zero), 0 to close the socket
