@@ -26,6 +26,7 @@
 #include <czmq.h>
 #include <ip_utils.h>
 
+#include "celix_compiler.h"
 #include "pubsub_serializer_handler.h"
 #include "pubsub_endpoint.h"
 #include "pubsub_serializer.h"
@@ -137,7 +138,7 @@ pubsub_zmq_admin_t* pubsub_zmqAdmin_create(celix_bundle_context_t *ctx, celix_lo
 
     if (ip == NULL) {
         L_WARN("[PSA_ZMQ] Could not determine IP address for PSA, using default ip (%s)", PUBSUB_ZMQ_DEFAULT_IP);
-        ip = strndup(PUBSUB_ZMQ_DEFAULT_IP, 1024);
+        ip = strdup(PUBSUB_ZMQ_DEFAULT_IP);
     }
 
     psa->ipAddress = ip;
@@ -160,7 +161,7 @@ pubsub_zmq_admin_t* pubsub_zmqAdmin_create(celix_bundle_context_t *ctx, celix_lo
     long nrThreads = celix_bundleContext_getPropertyAsLong(ctx, PUBSUB_ZMQ_NR_THREADS_KEY, 0);
     if (nrThreads > 0) {
         zsys_set_io_threads((size_t)nrThreads);
-        L_INFO("[PSA_ZMQ] Using %d threads for ZMQ", (size_t)nrThreads);
+        L_INFO("[PSA_ZMQ] Using %ld threads for ZMQ", nrThreads);
     }
 
 
@@ -483,7 +484,7 @@ celix_status_t pubsub_zmqAdmin_teardownTopicSender(void *handle, const char *sco
     return status;
 }
 
-celix_status_t pubsub_zmqAdmin_setupTopicReceiver(void *handle, const char *scope, const char *topic, const celix_properties_t *topicProperties, long serializerSvcId __attribute__((unused)), long protocolSvcId, celix_properties_t **outSubscriberEndpoint) {
+celix_status_t pubsub_zmqAdmin_setupTopicReceiver(void *handle, const char *scope, const char *topic, const celix_properties_t *topicProperties, long serializerSvcId CELIX_UNUSED, long protocolSvcId, celix_properties_t **outSubscriberEndpoint) {
     pubsub_zmq_admin_t *psa = handle;
     celix_properties_t *newEndpoint = NULL;
 
@@ -660,7 +661,7 @@ celix_status_t pubsub_zmqAdmin_removeDiscoveredEndpoint(void *handle, const celi
     return status;
 }
 
-bool pubsub_zmqAdmin_executeCommand(void *handle, const char *commandLine, FILE *out, FILE *errStream __attribute__((unused))) {
+bool pubsub_zmqAdmin_executeCommand(void *handle, const char *commandLine, FILE *out, FILE *errStream CELIX_UNUSED) {
     pubsub_zmq_admin_t *psa = handle;
     celix_status_t  status = CELIX_SUCCESS;
 
@@ -796,4 +797,4 @@ static celix_status_t zmq_getIpAddress(const char* interface, char** ip) {
 
     return status;
 }
-#endif
\ No newline at end of file
+#endif
