@@ -25,6 +25,7 @@
 #include <czmq.h>
 #include <uuid/uuid.h>
 
+#include "celix_compiler.h"
 #include "celix_utils.h"
 #include "pubsub_constants.h"
 #include "pubsub/publisher.h"
@@ -112,8 +113,6 @@ typedef struct psa_zmq_zerocopy_free_entry {
 static void* psa_zmq_getPublisherService(void *handle, const celix_bundle_t *requestingBundle, const celix_properties_t *svcProperties);
 static void psa_zmq_ungetPublisherService(void *handle, const celix_bundle_t *requestingBundle, const celix_properties_t *svcProperties);
 static unsigned int rand_range(unsigned int min, unsigned int max);
-static void delay_first_send_for_late_joiners(pubsub_zmq_topic_sender_t *sender);
-
 static int psa_zmq_topicPublicationSend(void* handle, unsigned int msgTypeId, const void *msg, celix_properties_t *metadata);
 
 pubsub_zmq_topic_sender_t* pubsub_zmqTopicSender_create(
@@ -347,7 +346,7 @@ static int psa_zmq_localMsgTypeIdForMsgType(void* handle, const char* msgType, u
     return 0;
 }
 
-static void* psa_zmq_getPublisherService(void *handle, const celix_bundle_t *requestingBundle, const celix_properties_t *svcProperties __attribute__((unused))) {
+static void* psa_zmq_getPublisherService(void *handle, const celix_bundle_t *requestingBundle, const celix_properties_t *svcProperties CELIX_UNUSED) {
     pubsub_zmq_topic_sender_t *sender = handle;
     long bndId = celix_bundle_getId(requestingBundle);
 
@@ -370,7 +369,7 @@ static void* psa_zmq_getPublisherService(void *handle, const celix_bundle_t *req
     return &entry->service;
 }
 
-static void psa_zmq_ungetPublisherService(void *handle, const celix_bundle_t *requestingBundle, const celix_properties_t *svcProperties __attribute__((unused))) {
+static void psa_zmq_ungetPublisherService(void *handle, const celix_bundle_t *requestingBundle, const celix_properties_t *svcProperties CELIX_UNUSED) {
     pubsub_zmq_topic_sender_t *sender = handle;
     long bndId = celix_bundle_getId(requestingBundle);
 
@@ -424,8 +423,6 @@ static int psa_zmq_topicPublicationSend(void* handle, unsigned int msgTypeId, co
         return status;
     }
 
-    delay_first_send_for_late_joiners(sender);
-
     // Some ZMQ functions are not thread-safe, but this atomic compare exchange ensures one access at a time.
     // Also protect sender->zmqBuffers (header, meta and footer)
     bool expected = false;
@@ -437,14 +434,16 @@ static int psa_zmq_topicPublicationSend(void* handle, unsigned int msgTypeId, co
     pubsub_protocol_message_t message;
     message.payload.payload = serializedIoVecOutput->iov_base;
     message.payload.length = serializedIoVecOutput->iov_len;
+    message.header.convertEndianess = 0;
 
     void *payloadData = NULL;
     size_t payloadLength = 0;
     sender->protocol->encodePayload(sender->protocol->handle, &message, &payloadData, &payloadLength);
 
+    size_t metadataSize = 0;
     if (metadata != NULL) {
         message.metadata.metadata = metadata;
-        sender->protocol->encodeMetadata(sender->protocol->handle, &message, &sender->zmqBuffers.metadataBuffer, &sender->zmqBuffers.metadataBufferSize);
+        sender->protocol->encodeMetadata(sender->protocol->handle, &message, &sender->zmqBuffers.metadataBuffer, &sender->zmqBuffers.metadataBufferSize, &metadataSize);
     } else {
         message.metadata.metadata = NULL;
     }
@@ -456,11 +455,10 @@ static int psa_zmq_topicPublicationSend(void* handle, unsigned int msgTypeId, co
     message.header.msgMajorVersion = majorVersion;
     message.header.msgMinorVersion = minorversion;
     message.header.payloadSize = payloadLength;
-    message.header.metadataSize = sender->zmqBuffers.metadataBufferSize;
+    message.header.metadataSize = metadataSize;
     message.header.payloadPartSize = payloadLength;
     message.header.payloadOffset = 0;
     message.header.isLastSegment = 1;
-    message.header.convertEndianess = 0;
 
     sender->protocol->encodeHeader(sender->protocol->handle, &message, &sender->zmqBuffers.headerBuffer, &sender->zmqBuffers.headerBufferSize);
 
@@ -488,7 +486,7 @@ static int psa_zmq_topicPublicationSend(void* handle, unsigned int msgTypeId, co
 
         //send Payload
         if (rc > 0) {
-            int flag = ((sender->zmqBuffers.metadataBufferSize > 0)  || (sender->zmqBuffers.footerBufferSize > 0)) ? ZMQ_SNDMORE : 0;
+            int flag = ((metadataSize > 0)  || (sender->zmqBuffers.footerBufferSize > 0)) ? ZMQ_SNDMORE : 0;
             zmq_msg_init_data(&msg2, payloadData, payloadLength, psa_zmq_freeMsg, freeMsgEntry);
             rc = zmq_msg_send(&msg2, socket, flag);
             if (rc == -1) {
@@ -498,9 +496,9 @@ static int psa_zmq_topicPublicationSend(void* handle, unsigned int msgTypeId, co
         }
 
         //send MetaData
-        if (rc > 0 && sender->zmqBuffers.metadataBufferSize > 0) {
+        if (rc > 0 && metadataSize > 0) {
             int flag = (sender->zmqBuffers.footerBufferSize > 0 ) ? ZMQ_SNDMORE : 0;
-            zmq_msg_init_data(&msg3, sender->zmqBuffers.metadataBuffer, sender->zmqBuffers.metadataBufferSize, NULL, NULL);
+            zmq_msg_init_data(&msg3, sender->zmqBuffers.metadataBuffer, metadataSize, NULL, NULL);
             rc = zmq_msg_send(&msg3, socket, flag);
             if (rc == -1) {
                 L_WARN("Error sending metadata msg. %s", strerror(errno));
@@ -523,8 +521,8 @@ static int psa_zmq_topicPublicationSend(void* handle, unsigned int msgTypeId, co
         zmsg_t *msg = zmsg_new();
         zmsg_addmem(msg, sender->zmqBuffers.headerBuffer, sender->zmqBuffers.headerBufferSize);
         zmsg_addmem(msg, payloadData, payloadLength);
-        if (sender->zmqBuffers.metadataBufferSize > 0) {
-            zmsg_addmem(msg, sender->zmqBuffers.metadataBuffer, sender->zmqBuffers.metadataBufferSize);
+        if (metadataSize > 0) {
+            zmsg_addmem(msg, sender->zmqBuffers.metadataBuffer, metadataSize);
         }
         if (sender->zmqBuffers.footerBufferSize > 0) {
             zmsg_addmem(msg, sender->zmqBuffers.footerBuffer, sender->zmqBuffers.footerBufferSize);
@@ -556,17 +554,6 @@ static int psa_zmq_topicPublicationSend(void* handle, unsigned int msgTypeId, co
     return status;
 }
 
-static void delay_first_send_for_late_joiners(pubsub_zmq_topic_sender_t *sender) {
-
-    static bool firstSend = true;
-
-    if (firstSend) {
-        L_INFO("PSA_ZMQ_TP: Delaying first send for late joiners...\n");
-        sleep(FIRST_SEND_DELAY_IN_SECONDS);
-        firstSend = false;
-    }
-}
-
 static unsigned int rand_range(unsigned int min, unsigned int max) {
     double scaled = ((double)random())/((double)RAND_MAX);
     return (unsigned int)((max-min+1)*scaled + min);
