@@ -25,6 +25,7 @@
 #include <string.h>
 
 #include "pubsub_message_serialization_marker.h"
+#include "celix_compiler.h"
 #include "celix_constants.h"
 #include "dyn_function.h"
 #include "celix_version.h"
@@ -35,6 +36,7 @@
 #include "celix_log_helper.h"
 #include "pubsub_message_serialization_service.h"
 #include "celix_shell_command.h"
+#include "celix_threads.h"
 
 #define MAX_PATH_LEN    1024
 
@@ -63,7 +65,7 @@ struct pubsub_serialization_provider {
     //serialization callbacks
     celix_status_t (*serialize)(pubsub_serialization_entry_t* entry, const void* msg, struct iovec** output, size_t* outputIovLen);
     void (*freeSerializeMsg)(pubsub_serialization_entry_t* entry, struct iovec* input, size_t inputIovLen);
-    celix_status_t (*deserialize)(pubsub_serialization_entry_t* entry, const struct iovec* input, size_t inputIovLen __attribute__((unused)), void **out);
+    celix_status_t (*deserialize)(pubsub_serialization_entry_t* entry, const struct iovec* input, size_t inputIovLen CELIX_UNUSED, void **out);
     void (*freeDeserializeMsg)(pubsub_serialization_entry_t* entry, void *msg);
 
     //updated serialization services
@@ -168,7 +170,7 @@ static FILE* openFileStream(pubsub_serialization_provider_t* provider, descripto
     return result;
 }
 
-static unsigned int pubsub_serializationProvider_getMsgId(pubsub_serialization_provider_t* provider __attribute__((unused)), dyn_message_type *msg) {
+static unsigned int pubsub_serializationProvider_getMsgId(pubsub_serialization_provider_t* provider CELIX_UNUSED, dyn_message_type *msg) {
     unsigned int msgId = 0;
 
     char *msgName = NULL;
@@ -454,7 +456,7 @@ static void pubsub_serializationProvider_parseDescriptors(pubsub_serialization_p
 
         bool unique = pubsub_serializationProvider_validateEntry(provider, serEntry);
         if (unique && serEntry->valid) { //note only register if unique and valid
-            L_DEBUG("Adding message serialization entry for msg %s with id %d and version %s", serEntry->msgFqn, serEntry->msgId, serEntry->msgVersion);
+            L_DEBUG("Adding message serialization entry for msg %s with id %d and version %s", serEntry->msgFqn, serEntry->msgId, serEntry->msgVersionStr);
             pubsub_serializationProvider_registerSerializationEntry(provider, serEntry);
         }
 
@@ -494,7 +496,7 @@ static void pubsub_serializationProvider_printEntryDetails(pubsub_serialization_
     free(bndName);
 }
 
-bool pubsub_serializationProvider_executeCommand(void *handle, const char *commandLine , FILE *outStream, FILE *errorStream  __attribute__((unused))) {
+bool pubsub_serializationProvider_executeCommand(void *handle, const char *commandLine , FILE *outStream, FILE *errorStream  CELIX_UNUSED) {
     pubsub_serialization_provider_t* provider = handle;
 
     bool verbose = false;
@@ -594,7 +596,7 @@ pubsub_serialization_provider_t *pubsub_serializationProvider_create(
         long serializationServiceRanking,
         celix_status_t (*serialize)(pubsub_serialization_entry_t* entry, const void* msg, struct iovec** output, size_t* outputIovLen),
         void (*freeSerializeMsg)(pubsub_serialization_entry_t* entry, struct iovec* input, size_t inputIovLen),
-        celix_status_t (*deserialize)(pubsub_serialization_entry_t* entry, const struct iovec* input, size_t inputIovLen __attribute__((unused)), void **out),
+        celix_status_t (*deserialize)(pubsub_serialization_entry_t* entry, const struct iovec* input, size_t inputIovLen CELIX_UNUSED, void **out),
         void (*freeDeserializeMsg)(pubsub_serialization_entry_t* entry, void *msg)) {
     pubsub_serialization_provider_t* provider = calloc(1, sizeof(*provider));
     provider->ctx = ctx;
