@@ -21,12 +21,16 @@
 #include "pubsub_serializer_handler.h"
 
 #include <string.h>
+#include <stdlib.h>
 
 #include "pubsub_message_serialization_marker.h"
 #include "celix_version.h"
 #include "pubsub_message_serialization_service.h"
 #include "celix_log_helper.h"
+#include "celix_compiler.h"
 #include "celix_constants.h"
+#include "celix_threads.h"
+#include "celix_utils.h"
 
 #define L_DEBUG(...) \
     celix_logHelper_debug(handler->logHelper, __VA_ARGS__)
@@ -138,7 +142,7 @@ struct pubsub_serializerHandler_callback_data {
     pubsub_serializer_handler_t* handler;
 };
 
-static void pubsub_serializerHandler_useMarkerSvcCallback(void *handle, void* svc __attribute__((unused)), const celix_properties_t* props) {
+static void pubsub_serializerHandler_useMarkerSvcCallback(void *handle, void* svc CELIX_UNUSED, const celix_properties_t* props) {
     struct pubsub_serializerHandler_callback_data* data = handle;
     const char* serType = celix_properties_get(props, PUBSUB_MESSAGE_SERIALIZATION_MARKER_SERIALIZATION_TYPE_PROPERTY, NULL);
     bool backwardsCompatible = celix_properties_getAsBool(props, PUBSUB_MESSAGE_SERIALIZATION_MARKER_SERIALIZATION_BACKWARDS_COMPATIBLE, false);
@@ -216,7 +220,7 @@ void pubsub_serializerHandler_addSerializationService(pubsub_serializer_handler_
 
     celix_version_t* msgVersion = celix_version_createVersionFromString(version);
     if (msgVersion == NULL) {
-        L_ERROR("%s service has an invalid %s property. value is '%s'", PUBSUB_MESSAGE_SERIALIZATION_SERVICE_NAME, PUBSUB_MESSAGE_SERIALIZATION_SERVICE_MSG_VERSION_PROPERTY, msgVersion);
+        L_ERROR("%s service has an invalid %s property. value is '%s'", PUBSUB_MESSAGE_SERIALIZATION_SERVICE_NAME, PUBSUB_MESSAGE_SERIALIZATION_SERVICE_MSG_VERSION_PROPERTY, version);
         return;
     }
 
