@@ -28,10 +28,12 @@
 #include <uuid/uuid.h>
 #include <curl/curl.h>
 #include <limits.h>
+#include <net/if.h>
 
 #include <jansson.h>
 #include "json_serializer.h"
 #include "utils.h"
+#include "celix_utils.h"
 
 #include "import_registration_dfi.h"
 #include "export_registration_dfi.h"
@@ -68,6 +70,7 @@
  */
 #define CELIX_RSA_USE_STOP_EXPORT_THREAD true
 
+
 struct remote_service_admin {
     celix_bundle_context_t *context;
     celix_log_helper_t *loghelper;
@@ -87,6 +90,7 @@ struct remote_service_admin {
 
     char *port;
     char *ip;
+    char *discoveryInterface;
 
     struct mg_context *ctx;
 
@@ -129,9 +133,9 @@ static int remoteServiceAdmin_callback(struct mg_connection *conn);
 static celix_status_t remoteServiceAdmin_createEndpointDescription(remote_service_admin_t *admin, service_reference_pt reference, celix_properties_t *props, char *interface, endpoint_description_t **description);
 static celix_status_t remoteServiceAdmin_send(void *handle, endpoint_description_t *endpointDescription, char *request, celix_properties_t *metadata, char **reply, int* replyStatus);
 static celix_status_t remoteServiceAdmin_getIpAddress(char* interface, char** ip);
+static char* remoteServiceAdmin_getIFNameForIP(const char *ip);
 static size_t remoteServiceAdmin_readCallback(void *ptr, size_t size, size_t nmemb, void *userp);
 static size_t remoteServiceAdmin_write(void *contents, size_t size, size_t nmemb, void *userp);
-static void remoteServiceAdmin_log(remote_service_admin_t *admin, int level, const char *file, int line, const char *msg, ...);
 static void remoteServiceAdmin_setupStopExportsThread(remote_service_admin_t* admin);
 static void remoteServiceAdmin_teardownStopExportsThread(remote_service_admin_t* admin);
 
@@ -192,24 +196,36 @@ celix_status_t remoteServiceAdmin_create(celix_bundle_context_t *context, remote
          celixThreadMutex_create(&(*admin)->importedServicesLock, NULL);
 
         (*admin)->loghelper = celix_logHelper_create(context, "celix_rsa_admin");
-        dynCommon_logSetup((void *)remoteServiceAdmin_log, *admin, 1);
-        dynType_logSetup((void *)remoteServiceAdmin_log, *admin, 1);
-        dynFunction_logSetup((void *)remoteServiceAdmin_log, *admin, 1);
-        dynInterface_logSetup((void *)remoteServiceAdmin_log, *admin, 1);
-        jsonSerializer_logSetup((void *)remoteServiceAdmin_log, *admin, 1);
-        jsonRpc_logSetup((void *)remoteServiceAdmin_log, *admin, 1);
 
         long port = celix_bundleContext_getPropertyAsLong(context, RSA_PORT_KEY, RSA_PORT_DEFAULT);
         const char *ip = celix_bundleContext_getProperty(context, RSA_IP_KEY, RSA_IP_DEFAULT);
         const char *interface = celix_bundleContext_getProperty(context, RSA_INTERFACE_KEY, NULL);
         (*admin)->curlShareEnabled = celix_bundleContext_getPropertyAsBool(context, RSA_DFI_USE_CURL_SHARE_HANDLE, RSA_DFI_USE_CURL_SHARE_HANDLE_DEFAULT);
 
+        const char *networkInterfaces = celix_bundleContext_getProperty(context, CELIX_RSA_NETWORK_INTERFACES, NULL);
+        char *interfacesCopy = NULL;
+        if (networkInterfaces != NULL) {
+            interfacesCopy = celix_utils_strdup(networkInterfaces);
+            const char delimiter[2] = ",";
+            char *savePtr;
+            //TODO Supports multiple network interfaces
+            interface = strtok_r(interfacesCopy, delimiter, &savePtr);
+        }
+
+        char *discoveryInterface = NULL;
         char *detectedIp = NULL;
         if ((interface != NULL) && (remoteServiceAdmin_getIpAddress((char*)interface, &detectedIp) != CELIX_SUCCESS)) {
             celix_logHelper_log((*admin)->loghelper, CELIX_LOG_LEVEL_WARNING, "RSA: Could not retrieve IP address for interface %s", interface);
         }
         if (detectedIp != NULL) {
             ip = detectedIp;
+            discoveryInterface = celix_utils_strdup(interface);
+        } else {
+            discoveryInterface = remoteServiceAdmin_getIFNameForIP(ip);
+        }
+
+        if (interfacesCopy != NULL) {
+            free(interfacesCopy);
         }
 
         if (ip != NULL) {
@@ -282,6 +298,12 @@ celix_status_t remoteServiceAdmin_create(celix_bundle_context_t *context, remote
 
         } while (((*admin)->ctx == NULL) && (port_counter < MAX_NUMBER_OF_RESTARTS));
 
+        if (bindToAllInterfaces) {
+            free(discoveryInterface);
+            (*admin)->discoveryInterface = celix_utils_strdup("all");//announce service to all network interface
+        } else {
+            (*admin)->discoveryInterface = discoveryInterface;
+        }
     }
 
     bool logCalls = celix_bundleContext_getPropertyAsBool(context, RSA_LOG_CALLS_KEY, RSA_LOG_CALLS_DEFAULT);
@@ -309,7 +331,7 @@ celix_status_t remoteServiceAdmin_destroy(remote_service_admin_t **admin) {
     if ( (*admin)->logFile != NULL && (*admin)->logFile != stdout) {
         fclose((*admin)->logFile);
     }
-
+    free((*admin)->discoveryInterface);
     free((*admin)->ip);
     free((*admin)->port);
     curl_share_cleanup((*admin)->curlShare);
@@ -390,13 +412,13 @@ celix_status_t remoteServiceAdmin_stop(remote_service_admin_t *admin) {
 
     hash_map_iterator_pt iter = hashMapIterator_create(admin->exportedServices);
     while (hashMapIterator_hasNext(iter)) {
-        array_list_pt exports = hashMapIterator_nextValue(iter);
+        celix_array_list_t *exports = hashMapIterator_nextValue(iter);
         int i;
-        for (i = 0; i < arrayList_size(exports); i++) {
-            export_registration_t *export = arrayList_get(exports, i);
+        for (i = 0; i < celix_arrayList_size(exports); i++) {
+            export_registration_t *export = celix_arrayList_get(exports, i);
             remoteServiceAdmin_stopExport(admin, export);
         }
-        arrayList_destroy(exports);
+        celix_arrayList_destroy(exports);
     }
     hashMapIterator_destroy(iter);
     celixThreadRwlock_unlock(&admin->exportedServicesLock);
@@ -441,14 +463,14 @@ static int remoteServiceAdmin_callback(struct mg_connection *conn) {
     celix_properties_t *metadata = NULL;
 
     const struct mg_request_info *request_info = mg_get_request_info(conn);
-    if (request_info->uri != NULL) {
+    if (request_info->request_uri != NULL) {
         remote_service_admin_t *rsa = request_info->user_data;
 
 
-        if (strncmp(request_info->uri, "/service/", 9) == 0 && strcmp("POST", request_info->request_method) == 0) {
+        if (strncmp(request_info->request_uri, "/service/", 9) == 0 && strcmp("POST", request_info->request_method) == 0) {
 
             // uri = /services/myservice/call
-            const char *uri = request_info->uri;
+            const char *uri = request_info->request_uri;
             // rest = myservice/call
 
             const char *rest = uri+9;
@@ -475,10 +497,10 @@ static int remoteServiceAdmin_callback(struct mg_connection *conn) {
             hash_map_iterator_pt iter = hashMapIterator_create(rsa->exportedServices);
             while (hashMapIterator_hasNext(iter)) {
                 hash_map_entry_pt entry = hashMapIterator_nextEntry(iter);
-                array_list_pt exports = hashMapEntry_getValue(entry);
+                celix_array_list_t *exports = hashMapEntry_getValue(entry);
                 int expIt = 0;
-                for (expIt = 0; expIt < arrayList_size(exports); expIt++) {
-                    export_registration_t *check = arrayList_get(exports, expIt);
+                for (expIt = 0; expIt < celix_arrayList_size(exports); expIt++) {
+                    export_registration_t *check = celix_arrayList_get(exports, expIt);
                     export_reference_t * ref = NULL;
                     exportRegistration_getExportReference(check, &ref);
                     endpoint_description_t * checkEndpoint = NULL;
@@ -555,7 +577,7 @@ static int remoteServiceAdmin_callback(struct mg_connection *conn) {
     return result;
 }
 
-celix_status_t remoteServiceAdmin_exportService(remote_service_admin_t *admin, char *serviceId, celix_properties_t *properties, array_list_pt *registrations) {
+celix_status_t remoteServiceAdmin_exportService(remote_service_admin_t *admin, char *serviceId, celix_properties_t *properties, array_list_pt *registrationsOut) {
     celix_status_t status = CELIX_SUCCESS;
 
     bool export = false;
@@ -569,7 +591,7 @@ celix_status_t remoteServiceAdmin_exportService(remote_service_admin_t *admin, c
 
         token = strtok_r(ecCopy, delimiter, &savePtr);
         while (token != NULL) {
-            if (strncmp(utils_stringTrim(token), RSA_DFI_CONFIGURATION_TYPE, 1024) == 0) {
+            if (strncmp(celix_utils_trimInPlace(token), RSA_DFI_CONFIGURATION_TYPE, 1024) == 0) {
                 export = true;
                 break;
             }
@@ -582,8 +604,9 @@ celix_status_t remoteServiceAdmin_exportService(remote_service_admin_t *admin, c
         export = true;
     }
 
+    celix_array_list_t *registrations = NULL;
     if (export) {
-        arrayList_create(registrations);
+        registrations = celix_arrayList_create();
         array_list_pt references = NULL;
         service_reference_pt reference = NULL;
         char filter[256];
@@ -636,22 +659,34 @@ celix_status_t remoteServiceAdmin_exportService(remote_service_admin_t *admin, c
             if (status == CELIX_SUCCESS) {
                 status = exportRegistration_start(registration);
                 if (status == CELIX_SUCCESS) {
-                    arrayList_add(*registrations, registration);
+                    celix_arrayList_add(registrations, registration);
                 }
             }
         }
 
 
-        if (status == CELIX_SUCCESS) {
+        if (status == CELIX_SUCCESS && celix_arrayList_size(registrations) > 0) {
             celixThreadRwlock_writeLock(&admin->exportedServicesLock);
-            hashMap_put(admin->exportedServices, reference, *registrations);
+            hashMap_put(admin->exportedServices, reference, registrations);
             celixThreadRwlock_unlock(&admin->exportedServicesLock);
         } else {
-            arrayList_destroy(*registrations);
-            *registrations = NULL;
+            celix_arrayList_destroy(registrations);
+            registrations = NULL;
         }
     }
 
+    if (status == CELIX_SUCCESS) {
+        //We return a empty list of registrations if Remote Service Admin does not recognize any of the configuration types.
+    	celix_array_list_t *newRegistrations = celix_arrayList_create();
+        if (registrations != NULL) {
+            int regSize = celix_arrayList_size(registrations);
+            for (int i = 0; i < regSize; ++i) {
+                celix_arrayList_add(newRegistrations, celix_arrayList_get(registrations, i));
+            }
+        }
+        *registrationsOut = newRegistrations;
+    }
+
     return status;
 }
 
@@ -668,9 +703,13 @@ celix_status_t remoteServiceAdmin_removeExportedService(remote_service_admin_t *
         celixThreadRwlock_writeLock(&admin->exportedServicesLock);
         exportReference_getExportedService(ref, &servRef);
 
-        array_list_pt exports = (array_list_pt)hashMap_remove(admin->exportedServices, servRef);
-        if(exports!=NULL){
-            arrayList_destroy(exports);
+        celix_array_list_t *exports = (celix_array_list_t *)hashMap_get(admin->exportedServices, servRef);
+        if (exports != NULL) {
+            celix_arrayList_remove(exports, registration);
+            if (celix_arrayList_size(exports) == 0) {
+                hashMap_remove(admin->exportedServices, servRef);
+                celix_arrayList_destroy(exports);
+            }
         }
 
         remoteServiceAdmin_stopExport(admin, registration);
@@ -731,6 +770,9 @@ static celix_status_t remoteServiceAdmin_createEndpointDescription(remote_servic
     celix_properties_set(endpointProperties, OSGI_RSA_SERVICE_IMPORTED, "true");
     celix_properties_set(endpointProperties, OSGI_RSA_SERVICE_IMPORTED_CONFIGS, (char*) RSA_DFI_CONFIGURATION_TYPE);
     celix_properties_set(endpointProperties, RSA_DFI_ENDPOINT_URL, url);
+    if (admin->discoveryInterface != NULL) {
+        celix_properties_set(endpointProperties, CELIX_RSA_NETWORK_INTERFACES, admin->discoveryInterface);
+    }
 
     if (props != NULL) {
         hash_map_iterator_pt propIter = hashMapIterator_create(props);
@@ -746,11 +788,9 @@ static celix_status_t remoteServiceAdmin_createEndpointDescription(remote_servic
         status = CELIX_ENOMEM;
     } else {
         (*endpoint)->id = (char*) celix_properties_get(endpointProperties, (char*) OSGI_RSA_ENDPOINT_ID, NULL);
-        const char *serviceId = NULL;
-        serviceReference_getProperty(reference, (char*) OSGI_FRAMEWORK_SERVICE_ID, &serviceId);
-        (*endpoint)->serviceId = strtoull(serviceId, NULL, 0);
+        (*endpoint)->serviceId = serviceReference_getServiceId(reference);
         (*endpoint)->frameworkUUID = (char*) celix_properties_get(endpointProperties, (char*) OSGI_RSA_ENDPOINT_FRAMEWORK_UUID, NULL);
-        (*endpoint)->service = strndup(interface, 1024*10);
+        (*endpoint)->serviceName = strndup(interface, 1024*10);
         (*endpoint)->properties = endpointProperties;
     }
 
@@ -792,13 +832,36 @@ static celix_status_t remoteServiceAdmin_getIpAddress(char* interface, char** ip
     return status;
 }
 
+static char* remoteServiceAdmin_getIFNameForIP(const char *ip) {
+    struct ifaddrs *ifaddr, *ifa;
+    char host[NI_MAXHOST];
+    char *ifName = NULL;
+    if (ip != NULL && getifaddrs(&ifaddr) != -1)
+    {
+        for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next)
+        {
+            if (ifa->ifa_addr == NULL)
+                continue;
+
+            if ((getnameinfo(ifa->ifa_addr,sizeof(struct sockaddr_in), host, NI_MAXHOST, NULL, 0, NI_NUMERICHOST) == 0)) {
+                if (strcmp(host, ip) == 0) {
+                    ifName = celix_utils_strdup(ifa->ifa_name);
+                    break;
+                }
+            }
+        }
+
+        freeifaddrs(ifaddr);
+    }
+    return ifName;
+}
 
 celix_status_t remoteServiceAdmin_destroyEndpointDescription(endpoint_description_t **description)
 {
     celix_status_t status = CELIX_SUCCESS;
 
     celix_properties_destroy((*description)->properties);
-    free((*description)->service);
+    free((*description)->serviceName);
     free(*description);
 
     return status;
@@ -828,7 +891,7 @@ celix_status_t remoteServiceAdmin_importService(remote_service_admin_t *admin, e
 
         token = strtok_r(ecCopy, delimiter, &savePtr);
         while (token != NULL) {
-            if (strncmp(utils_stringTrim(token), RSA_DFI_CONFIGURATION_TYPE, 1024) == 0) {
+            if (strncmp(celix_utils_trimInPlace(token), RSA_DFI_CONFIGURATION_TYPE, 1024) == 0) {
                 importService = true;
                 break;
             }
@@ -848,7 +911,7 @@ celix_status_t remoteServiceAdmin_importService(remote_service_admin_t *admin, e
         const char *objectClass = celix_properties_get(endpointDescription->properties, "objectClass", NULL);
         const char *serviceVersion = celix_properties_get(endpointDescription->properties, CELIX_FRAMEWORK_SERVICE_VERSION, NULL);
 
-        celix_logHelper_log(admin->loghelper, CELIX_LOG_LEVEL_INFO, "RSA: Import service %s", endpointDescription->service);
+        celix_logHelper_log(admin->loghelper, CELIX_LOG_LEVEL_INFO, "RSA: Import service %s", endpointDescription->serviceName);
         celix_logHelper_log(admin->loghelper, CELIX_LOG_LEVEL_INFO, "Registering service factory (proxy) for service '%s'\n",
                       objectClass);
 
@@ -998,17 +1061,3 @@ static size_t remoteServiceAdmin_write(void *contents, size_t size, size_t nmemb
     return size * nmemb;
 }
 
-
-static void remoteServiceAdmin_log(remote_service_admin_t *admin, int level, const char *file, int line, const char *msg, ...) {
-    va_list ap;
-    va_start(ap, msg);
-    int levels[5] = {0, CELIX_LOG_LEVEL_ERROR, CELIX_LOG_LEVEL_WARNING, CELIX_LOG_LEVEL_INFO, CELIX_LOG_LEVEL_DEBUG};
-
-    char buf1[256];
-    snprintf(buf1, 256, "FILE:%s, LINE:%i, MSG:", file, line);
-
-    char buf2[256];
-    vsnprintf(buf2, 256, msg, ap);
-    celix_logHelper_log(admin->loghelper, levels[level], "%s%s", buf1, buf2);
-    va_end(ap);
-}
