@@ -9,8 +9,7 @@
  *
  *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * Unless required by applicable law or agreed to in writing,
- * software distributed under the License is distributed on an
+ * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an
  * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
  *  KIND, either express or implied.  See the License for the
  * specific language governing permissions and limitations
@@ -28,10 +27,11 @@
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
+#include <stdbool.h>
 
-#include "celixbool.h"
 #include "topology_manager.h"
 #include "bundle_context.h"
+#include "celix_compiler.h"
 #include "celix_constants.h"
 #include "bundle.h"
 #include "remote_service_admin.h"
@@ -70,7 +70,7 @@ struct topology_manager {
 
 celix_status_t topologyManager_exportScopeChanged(void *handle, char *service_name);
 celix_status_t topologyManager_importScopeChanged(void *handle, char *service_name);
-static celix_status_t topologyManager_notifyListenersEndpointAdded(topology_manager_pt manager, remote_service_admin_service_t *rsa, array_list_pt registrations);
+static celix_status_t topologyManager_notifyListenersEndpointAdded(topology_manager_pt manager, remote_service_admin_service_t *rsa, celix_array_list_t *registrations);
 static celix_status_t topologyManager_notifyListenersEndpointRemoved(topology_manager_pt manager, remote_service_admin_service_t *rsa, export_registration_t *export);
 
 static celix_status_t topologyManager_addImportedService_nolock(void *handle, endpoint_description_t *endpoint, char *matchedFilter);
@@ -146,7 +146,7 @@ celix_status_t topologyManager_closeImports(topology_manager_pt manager) {
 		hash_map_pt imports = hashMapEntry_getValue(entry);
 
 		if (imports != NULL) {
-			celix_logHelper_log(manager->loghelper, CELIX_LOG_LEVEL_INFO, "TOPOLOGY_MANAGER: Remove imported service (%s; %s).", ep->service, ep->id);
+			celix_logHelper_log(manager->loghelper, CELIX_LOG_LEVEL_INFO, "TOPOLOGY_MANAGER: Remove imported service (%s; %s).", ep->serviceName, ep->id);
 			hash_map_iterator_pt importsIter = hashMapIterator_create(imports);
 
 			while (hashMapIterator_hasNext(importsIter)) {
@@ -183,7 +183,7 @@ celix_status_t topologyManager_rsaAdding(void * handle, service_reference_pt ref
 	return status;
 }
 
-celix_status_t topologyManager_rsaAdded(void * handle, service_reference_pt unusedRef __attribute__((unused)), void * service) {
+celix_status_t topologyManager_rsaAdded(void * handle, service_reference_pt unusedRef CELIX_UNUSED, void * service) {
 	topology_manager_pt manager = (topology_manager_pt) handle;
 	celix_properties_t *serviceProperties = NULL;
 	remote_service_admin_service_t *rsa = (remote_service_admin_service_t *) service;
@@ -231,17 +231,12 @@ celix_status_t topologyManager_rsaAdded(void * handle, service_reference_pt unus
 
         scope_getExportProperties(manager->scope, reference, &serviceProperties);
 
-        array_list_pt endpoints = NULL;
+        celix_array_list_t *endpoints = NULL;
         celix_status_t status = rsa->exportService(rsa->admin, (char*)serviceId, serviceProperties, &endpoints);
 
         if (status == CELIX_SUCCESS) {
             hash_map_pt exports = hashMapEntry_getValue(entry);
-
-            if (exports == NULL) {
-                exports = hashMap_create(NULL, NULL, NULL, NULL);
-                hashMap_put(manager->exportedServices,reference,exports);
-            }
-
+            assert(exports != NULL);// It must not be NULL, It has been created in function topologyManager_addExportedService_nolock
             hashMap_put(exports, rsa, endpoints);
             topologyManager_notifyListenersEndpointAdded(manager, rsa, endpoints);
         }
@@ -273,36 +268,18 @@ celix_status_t topologyManager_rsaRemoved(void * handle, service_reference_pt re
 	while (hashMapIterator_hasNext(exportedSvcIter)) {
 
 		hash_map_entry_pt entry = hashMapIterator_nextEntry(exportedSvcIter);
-		service_reference_pt key = hashMapEntry_getKey(entry);
 		hash_map_pt exports = hashMapEntry_getValue(entry);
 
-		/*
-		 * the problem here is that also the rsa has a a list of
-		 * endpoints which is destroyed when closing the exportRegistration
-		 */
-		array_list_pt exports_list = hashMap_get(exports, rsa);
-
+		celix_array_list_t *exports_list = (celix_array_list_t *)hashMap_remove(exports, rsa);
 		if (exports_list != NULL) {
 			int exportsIter = 0;
-			int exportListSize = arrayList_size(exports_list);
+			int exportListSize = celix_arrayList_size(exports_list);
 			for (exportsIter = 0; exports_list != NULL && exportsIter < exportListSize; exportsIter++) {
-				export_registration_t *export = arrayList_get(exports_list, exportsIter);
+				export_registration_t *export = celix_arrayList_get(exports_list, exportsIter);
 				topologyManager_notifyListenersEndpointRemoved(manager, rsa, export);
 				rsa->exportRegistration_close(rsa->admin, export);
 			}
-		}
-
-		hashMap_remove(exports, rsa);
-		/*if(exports_list!=NULL){
-			arrayList_destroy(exports_list);
-		}*/
-
-		if (hashMap_size(exports) == 0) {
-			hashMap_remove(manager->exportedServices, key);
-			hashMap_destroy(exports, false, false);
-
-			hashMapIterator_destroy(exportedSvcIter);
-			exportedSvcIter = hashMapIterator_create(manager->exportedServices);
+			celix_arrayList_destroy(exports_list);
 		}
 	}
 	hashMapIterator_destroy(exportedSvcIter);
@@ -313,15 +290,12 @@ celix_status_t topologyManager_rsaRemoved(void * handle, service_reference_pt re
 		hash_map_entry_pt entry = hashMapIterator_nextEntry(importedSvcIter);
 		hash_map_pt imports = hashMapEntry_getValue(entry);
 
-		import_registration_t *import = hashMap_get(imports, rsa);
+		import_registration_t *import = (import_registration_t *)hashMap_remove(imports, rsa);
 
 		if (import != NULL) {
 			celix_status_t subStatus = rsa->importRegistration_close(rsa->admin, import);
-
-			if (subStatus == CELIX_SUCCESS) {
-				hashMap_remove(imports, rsa);
-			} else {
-				status = subStatus;
+			if (subStatus != CELIX_SUCCESS) {
+				celix_logHelper_error(manager->loghelper, "TOPOLOGY_MANAGER: Failed to close imported endpoint.");
 			}
 		}
 	}
@@ -343,7 +317,7 @@ celix_status_t topologyManager_exportScopeChanged(void *handle, char *filterStr)
 	const char* serviceId = NULL;
 	bool found;
 	celix_properties_t *props;
-	filter_pt filter = filter_create(filterStr);
+	celix_autoptr(celix_filter_t) filter = celix_filter_create(filterStr);
 
 	if (filter == NULL) {
 		printf("filter creating failed\n");
@@ -408,9 +382,6 @@ celix_status_t topologyManager_exportScopeChanged(void *handle, char *filterStr)
 	// should unlock until here ?, avoid srvRefs[i] is released during topologyManager_removeExportedService
 	celixThreadMutex_unlock(&manager->lock);
 
-
-	filter_destroy(filter);
-
 	return status;
 }
 
@@ -441,7 +412,7 @@ celix_status_t topologyManager_importScopeChanged(void *handle, char *service_na
 		status = topologyManager_removeImportedService_nolock(manager, endpoint, NULL);
 
 		if (status != CELIX_SUCCESS) {
-			celix_logHelper_log(manager->loghelper, CELIX_LOG_LEVEL_ERROR, "TOPOLOGY_MANAGER: Removal of imported service (%s; %s) failed.", endpoint->service, endpoint->id);
+			celix_logHelper_log(manager->loghelper, CELIX_LOG_LEVEL_ERROR, "TOPOLOGY_MANAGER: Removal of imported service (%s; %s) failed.", endpoint->serviceName, endpoint->id);
 		} else {
 			status = topologyManager_addImportedService_nolock(manager, endpoint, NULL);
 		}
@@ -457,11 +428,11 @@ static celix_status_t topologyManager_addImportedService_nolock(void *handle, en
 	celix_status_t status = CELIX_SUCCESS;
 	topology_manager_pt manager = handle;
 
-	celix_logHelper_log(manager->loghelper, CELIX_LOG_LEVEL_DEBUG, "TOPOLOGY_MANAGER: Add imported service (%s; %s).", endpoint->service, endpoint->id);
+	celix_logHelper_log(manager->loghelper, CELIX_LOG_LEVEL_DEBUG, "TOPOLOGY_MANAGER: Add imported service (%s; %s).", endpoint->serviceName, endpoint->id);
 
 	// We should not try to add imported services to a closed listener.
 	if (manager->closed) {
-		celix_logHelper_log(manager->loghelper, CELIX_LOG_LEVEL_TRACE,"TOPOLOGY_MANAGER: Endpointer listener will close, Ignore imported service (%s; %s).", endpoint->service, endpoint->id);
+		celix_logHelper_log(manager->loghelper, CELIX_LOG_LEVEL_TRACE,"TOPOLOGY_MANAGER: Endpointer listener will close, Ignore imported service (%s; %s).", endpoint->serviceName, endpoint->id);
 		return CELIX_SUCCESS;
 	}
 
@@ -505,7 +476,7 @@ static celix_status_t topologyManager_removeImportedService_nolock(void *handle,
 	celix_status_t status = CELIX_SUCCESS;
 	topology_manager_pt manager = handle;
 
-	celix_logHelper_log(manager->loghelper, CELIX_LOG_LEVEL_DEBUG, "TOPOLOGY_MANAGER: Remove imported service (%s; %s).", endpoint->service, endpoint->id);
+	celix_logHelper_log(manager->loghelper, CELIX_LOG_LEVEL_DEBUG, "TOPOLOGY_MANAGER: Remove imported service (%s; %s).", endpoint->serviceName, endpoint->id);
 
 	hash_map_iterator_pt iter = hashMapIterator_create(manager->importedServices);
 	while (hashMapIterator_hasNext(iter)) {
@@ -553,7 +524,7 @@ celix_status_t topologyManager_removeImportedService(void *handle, endpoint_desc
 	return status;
 }
 
-static celix_status_t topologyManager_addExportedService_nolock(void * handle, service_reference_pt reference, void * service __attribute__((unused))) {
+static celix_status_t topologyManager_addExportedService_nolock(void * handle, service_reference_pt reference, void * service CELIX_UNUSED) {
     topology_manager_pt manager = handle;
 	celix_status_t status = CELIX_SUCCESS;
     long serviceId = serviceReference_getServiceId(reference);
@@ -570,6 +541,7 @@ static celix_status_t topologyManager_addExportedService_nolock(void * handle, s
 
 	scope_getExportProperties(manager->scope, reference, &serviceProperties);
 	hash_map_pt exports = hashMap_create(NULL, NULL, NULL, NULL);
+	assert(exports != NULL);
 	hashMap_put(manager->exportedServices, reference, exports);
 
 	int size = celix_arrayList_size(manager->rsaList);
@@ -581,7 +553,7 @@ static celix_status_t topologyManager_addExportedService_nolock(void * handle, s
 	for (int iter = 0; iter < size; iter++) {
 		remote_service_admin_service_t *rsa = celix_arrayList_get(manager->rsaList, iter);
 
-		array_list_pt endpoints = NULL;
+		celix_array_list_t *endpoints = NULL;
 		celix_status_t substatus = rsa->exportService(rsa->admin, serviceIdStr, serviceProperties, &endpoints);
 
 		if (substatus == CELIX_SUCCESS) {
@@ -610,7 +582,7 @@ celix_status_t topologyManager_addExportedService(void * handle, service_referen
 	return status;
 }
 
-static celix_status_t topologyManager_removeExportedService_nolock(void * handle, service_reference_pt reference, void * service  __attribute__((unused))) {
+static celix_status_t topologyManager_removeExportedService_nolock(void * handle, service_reference_pt reference, void * service CELIX_UNUSED) {
     topology_manager_pt manager = handle;
 	celix_status_t status = CELIX_SUCCESS;
 	long serviceId = serviceReference_getServiceId(reference);
@@ -623,18 +595,17 @@ static celix_status_t topologyManager_removeExportedService_nolock(void * handle
 		while (hashMapIterator_hasNext(iter)) {
 			hash_map_entry_pt entry = hashMapIterator_nextEntry(iter);
 			remote_service_admin_service_t *rsa = hashMapEntry_getKey(entry);
-			array_list_pt exportRegistrations = hashMapEntry_getValue(entry);
-
-			int size = arrayList_size(exportRegistrations);
-
-			for (int exportsIter = 0; exportsIter < size; exportsIter++) {
-				export_registration_t *export = arrayList_get(exportRegistrations, exportsIter);
-				topologyManager_notifyListenersEndpointRemoved(manager, rsa, export);
-				rsa->exportRegistration_close(rsa->admin, export);
+			celix_array_list_t *exportRegistrations = hashMap_remove(exports, rsa);
+			if (exportRegistrations != NULL) {
+				int size = celix_arrayList_size(exportRegistrations);
+				for (int exportsIter = 0; exportsIter < size; exportsIter++) {
+					export_registration_t *export = celix_arrayList_get(exportRegistrations, exportsIter);
+					topologyManager_notifyListenersEndpointRemoved(manager, rsa, export);
+					rsa->exportRegistration_close(rsa->admin, export);
+				}
+				celix_arrayList_destroy(exportRegistrations);
 			}
 
-			hashMap_remove(exports, rsa);
-			//arrayList_destroy(exportRegistrations);
 			hashMapIterator_destroy(iter);
 			iter = hashMapIterator_create(exports);
 
@@ -709,7 +680,7 @@ celix_status_t topologyManager_endpointListenerAdded(void* handle, service_refer
 
 	serviceReference_getProperty(reference, OSGI_ENDPOINT_LISTENER_SCOPE, &scope);
 
-	filter_pt filter = filter_create(scope);
+	celix_autoptr(celix_filter_t) filter = celix_filter_create(scope);
 
 	hash_map_iterator_pt refIter = hashMapIterator_create(manager->exportedServices);
 
@@ -720,13 +691,13 @@ celix_status_t topologyManager_endpointListenerAdded(void* handle, service_refer
 		while (hashMapIterator_hasNext(rsaIter)) {
 			hash_map_entry_pt entry = hashMapIterator_nextEntry(rsaIter);
 			remote_service_admin_service_t *rsa = hashMapEntry_getKey(entry);
-			array_list_pt registrations = hashMapEntry_getValue(entry);
+			celix_array_list_t *registrations = hashMapEntry_getValue(entry);
 
-			int arrayListSize = arrayList_size(registrations);
+			int arrayListSize = celix_arrayList_size(registrations);
 			int cnt = 0;
 
 			for (; cnt < arrayListSize; cnt++) {
-				export_registration_t *export = arrayList_get(registrations, cnt);
+				export_registration_t *export = celix_arrayList_get(registrations, cnt);
 				endpoint_description_t *endpoint = NULL;
 
 				status = topologyManager_getEndpointDescriptionForExportRegistration(rsa, export, &endpoint);
@@ -746,8 +717,6 @@ celix_status_t topologyManager_endpointListenerAdded(void* handle, service_refer
 
 	celixThreadMutex_unlock(&manager->lock);
 
-	filter_destroy(filter);
-
 	return status;
 }
 
@@ -778,7 +747,7 @@ celix_status_t topologyManager_endpointListenerRemoved(void * handle, service_re
 	return status;
 }
 
-static celix_status_t topologyManager_notifyListenersEndpointAdded(topology_manager_pt manager, remote_service_admin_service_t *rsa, array_list_pt registrations) {
+static celix_status_t topologyManager_notifyListenersEndpointAdded(topology_manager_pt manager, remote_service_admin_service_t *rsa, celix_array_list_t *registrations) {
 	celix_status_t status = CELIX_SUCCESS;
 
 	hash_map_iterator_pt iter = hashMapIterator_create(manager->listenerList);
@@ -791,11 +760,11 @@ static celix_status_t topologyManager_notifyListenersEndpointAdded(topology_mana
 
 		status = bundleContext_getService(manager->context, reference, (void **) &epl);
 		if (status == CELIX_SUCCESS) {
-			filter_pt filter = filter_create(scope);
+			celix_autoptr(celix_filter_t) filter = celix_filter_create(scope);
 
-			int regSize = arrayList_size(registrations);
+			int regSize = celix_arrayList_size(registrations);
 			for (int regIt = 0; regIt < regSize; regIt++) {
-				export_registration_t *export = arrayList_get(registrations, regIt);
+				export_registration_t *export = celix_arrayList_get(registrations, regIt);
 				endpoint_description_t *endpoint = NULL;
 				celix_status_t substatus = topologyManager_getEndpointDescriptionForExportRegistration(rsa, export, &endpoint);
 				if (substatus == CELIX_SUCCESS) {
@@ -808,7 +777,6 @@ static celix_status_t topologyManager_notifyListenersEndpointAdded(topology_mana
 					status = substatus;
 				}
 			}
-			filter_destroy(filter);
 			bundleContext_ungetService(manager->context, reference, NULL);
 		}
 	}
