@@ -20,7 +20,9 @@
 #ifndef CELIX_SHELL_CONSTANTS_H_
 #define CELIX_SHELL_CONSTANTS_H_
 
+#include <stdbool.h>
+
 #define CELIX_SHELL_USE_ANSI_COLORS                 "SHELL_USE_ANSI_COLORS"
-#define CELIX_SHELL_USE_ANSI_COLORS_DEFAULT_VALUE   "true"
+#define CELIX_SHELL_USE_ANSI_COLORS_DEFAULT_VALUE   true
 
 #endif /* CELIX_SHELL_CONSTANTS_H_ */
