@@ -23,6 +23,7 @@
 
 #include "service_tracker.h"
 #include "celix/BundleContext.h"
+#include "celix_err.h"
 #include "celix_framework_factory.h"
 #include "celix_framework.h"
 #include "service_tracker_customizer.h"
@@ -33,10 +34,10 @@ public:
     static constexpr const char * const TEST_BND2_LOC = "" SIMPLE_TEST_BUNDLE2_LOCATION "";
 
     CxxBundleContextTestSuite() {
-        auto* properties = properties_create();
-        properties_set(properties, "LOGHELPER_ENABLE_STDOUT_FALLBACK", "true");
-        properties_set(properties, "org.osgi.framework.storage.clean", "onFirstInit");
-        properties_set(properties, "org.osgi.framework.storage", ".cacheCxxBundleContextTestFramework");
+        auto* properties = celix_properties_create();
+        celix_properties_set(properties, "LOGHELPER_ENABLE_STDOUT_FALLBACK", "true");
+        celix_properties_set(properties, "org.osgi.framework.storage.clean", "onFirstInit");
+        celix_properties_set(properties, "CELIX_FRAMEWORK_CONDITION_SERVICES_ENABLED", "false");
 
         auto* cfw = celix_frameworkFactory_createFramework(properties);
         fw = std::shared_ptr<celix_framework_t>{cfw, [](celix_framework_t* f){ celix_frameworkFactory_destroyFramework(f); }};
@@ -416,6 +417,12 @@ TEST_F(CxxBundleContextTestSuite, LoggingUsingContext) {
     ctx->logFatal("fatal");
 }
 
+TEST_F(CxxBundleContextTestSuite, LoggingTssErrorsTest) {
+    ctx->logTssErrors(CELIX_LOG_LEVEL_FATAL);
+    celix_err_push("Hello ERR");
+    ctx->logTssErrors(CELIX_LOG_LEVEL_FATAL);
+}
+
 TEST_F(CxxBundleContextTestSuite, GetFramework) {
     EXPECT_FALSE(ctx->getFramework()->getUUID().empty());
     EXPECT_EQ(ctx->getFramework()->getFrameworkBundleContext()->getBundle().getId(), 0);
@@ -693,6 +700,7 @@ TEST_F(CxxBundleContextTestSuite, GetBundleInformation) {
     EXPECT_EQ(ctx->getBundle().getName(), std::string{"Celix Framework"});
     EXPECT_EQ(ctx->getBundle().getGroup(), std::string{"Celix/Framework"});
     EXPECT_EQ(ctx->getBundle().getDescription(), std::string{"The Celix Framework System Bundle"});
+    EXPECT_EQ(ctx->getBundle().getLocation(), std::string{""});
 
     std::atomic<bool> startCalled{false};
     auto bndTracker = ctx->trackBundles()
@@ -701,7 +709,9 @@ TEST_F(CxxBundleContextTestSuite, GetBundleInformation) {
             EXPECT_EQ(bnd.getName(), std::string{"Simple Test Bundle"});
             EXPECT_EQ(bnd.getGroup(), std::string{"test/group"});
             EXPECT_EQ(bnd.getDescription(), std::string{"Test Description"});
+            EXPECT_TRUE(strstr(bnd.getLocation().c_str(), ".zip") != nullptr);
             EXPECT_TRUE(!bnd.getEntry("META-INF/MANIFEST.MF").empty());
+            EXPECT_EQ(bnd.getEntry("/META-INF/MANIFEST.MF"), bnd.getEntry("META-INF/MANIFEST.MF"));
             EXPECT_EQ(bnd.getEntry("does-not-exist"), std::string{});
             EXPECT_EQ(bnd.getManifestValue("Bundle-SymbolicName"), std::string{"simple_test_bundle1"});
             EXPECT_EQ(bnd.getManifestValue("non-existing"), std::string{});
@@ -715,6 +725,7 @@ TEST_F(CxxBundleContextTestSuite, GetBundleInformation) {
     EXPECT_TRUE(startCalled);
 }
 
+#if __cplusplus >= 201703L //C++17 or higher
 class TestInterfaceWithStaticInfo {
 public:
     static constexpr std::string_view NAME = "TestName";
@@ -727,7 +738,7 @@ TEST_F(CxxBundleContextTestSuite, RegisterServiceWithNameAndVersionInfo) {
     EXPECT_EQ(reg->getServiceName(), "TestName");
     EXPECT_EQ(reg->getServiceVersion(), "1.2.3");
 }
-
+#endif
 
 TEST_F(CxxBundleContextTestSuite, listBundles) {
     auto list = ctx->listBundleIds();
