@@ -29,6 +29,7 @@
 #include "celix/ServiceRegistrationBuilder.h"
 #include "celix/UseServiceBuilder.h"
 #include "celix/TrackerBuilders.h"
+#include "celix/ScheduledEventBuilder.h"
 #include "celix/Bundle.h"
 #include "celix/Framework.h"
 
@@ -78,11 +79,19 @@ namespace celix {
          * @param name The optional name of the service. If not provided celix::typeName<I> will be used to defer the service name.
          * @return A ServiceRegistrationBuilder object.
          */
+#if __cplusplus >= 201703L //C++17 or higher
         template<typename I, typename Implementer>
         ServiceRegistrationBuilder<I> registerService(std::shared_ptr<Implementer> implementer, std::string_view name = {}) {
             std::shared_ptr<I> svc = implementer; //note Implement should be derived from I
             return ServiceRegistrationBuilder<I>{cCtx, std::move(svc), celix::typeName<I>(name)};
         }
+#else
+        template<typename I, typename Implementer>
+        ServiceRegistrationBuilder<I> registerService(std::shared_ptr<Implementer> implementer, const std::string& name = {}) {
+            std::shared_ptr<I> svc = implementer; //note Implement should be derived from I
+            return ServiceRegistrationBuilder<I>{cCtx, std::move(svc), celix::typeName<I>(name)};
+        }
+#endif
 
         /**
          * @brief Register a (unmanaged) service in the Celix framework using a fluent builder API.
@@ -94,11 +103,19 @@ namespace celix {
          * By default the service registration is configure to register the service async, but to unregister the
          * service sync (because the svc pointer is unmanaged).
          */
+#if __cplusplus >= 201703L //C++17 or higher
         template<typename I, typename Implementer>
         ServiceRegistrationBuilder<I> registerUnmanagedService(Implementer* svc, std::string_view name = {}) {
             auto unmanagedSvc = std::shared_ptr<I>{svc, [](I*){/*nop*/}};
             return ServiceRegistrationBuilder<I>{cCtx, std::move(unmanagedSvc), celix::typeName<I>(name), true, false};
         }
+#else
+        template<typename I, typename Implementer>
+        ServiceRegistrationBuilder<I> registerUnmanagedService(Implementer* svc, const std::string& name = {}) {
+            auto unmanagedSvc = std::shared_ptr<I>{svc, [](I*){/*nop*/}};
+            return ServiceRegistrationBuilder<I>{cCtx, std::move(unmanagedSvc), celix::typeName<I>(name), true, false};
+        }
+#endif
 
         //TODO registerServiceFactory<I>()
 
@@ -128,10 +145,17 @@ namespace celix {
          * @param name The optional service name to use. If not provided celix::typeName<I> will be used to defer the service name.
          * @return A UseServiceBuilder object.
          */
+#if __cplusplus >= 201703L //C++17 or higher
         template<typename I>
         UseServiceBuilder<I> useService(std::string_view name = {}) {
             return UseServiceBuilder<I>{cCtx, celix::typeName<I>(name), true};
         }
+#else
+        template<typename I>
+        UseServiceBuilder<I> useService(const std::string& name = {}) {
+            return UseServiceBuilder<I>{cCtx, celix::typeName<I>(name), true};
+        }
+#endif
 
         /**
          * @brief Use services registered in the Celix framework using a fluent builder API.
@@ -156,10 +180,17 @@ namespace celix {
          * @param name The optional service name to use. If not provided celix::typeName<I> will be used to defer the service name.
          * @return A UseServiceBuilder object.
          */
+#if __cplusplus >= 201703L //C++17 or higher
         template<typename I>
         UseServiceBuilder<I> useServices(std::string_view name = {}) {
             return UseServiceBuilder<I>{cCtx, celix::typeName<I>(name), false};
         }
+#else
+        template<typename I>
+        UseServiceBuilder<I> useServices(const std::string& name = {}) {
+            return UseServiceBuilder<I>{cCtx, celix::typeName<I>(name), false};
+        }
+#endif
 
         /**
          * @brief Finds the highest ranking service using the optional provided (LDAP) filter
@@ -172,10 +203,17 @@ namespace celix {
          * @param versionRange An optional version range.
          * @return The service id of the found service or -1 if the service was not found.
          */
+#if __cplusplus >= 201703L //C++17 or higher
         template<typename I>
         long findService(std::string_view filter = {}, std::string_view versionRange = {}) {
             return findServiceWithName(celix::typeName<I>(), filter, versionRange);
         }
+#else
+        template<typename I>
+        long findService(const std::string& filter = {}, const std::string& versionRange = {}) {
+            return findServiceWithName(celix::typeName<I>(), filter, versionRange);
+        }
+#endif
 
         /**
          * @brief Finds the highest ranking service using the provided service name and
@@ -186,6 +224,7 @@ namespace celix {
          * @param versionRange An optional version range.
          * @return The service id of the found service or -1 if the service was not found.
          */
+#if __cplusplus >= 201703L //C++17 or higher
         long findServiceWithName(std::string_view name, std::string_view filter = {}, std::string_view versionRange = {}) {
             waitIfAbleForEvents();
             celix_service_filter_options_t opts{};
@@ -194,6 +233,16 @@ namespace celix {
             opts.versionRange = versionRange.empty() ? nullptr : versionRange.data();
             return celix_bundleContext_findServiceWithOptions(cCtx.get(), &opts);
         }
+#else
+        long findServiceWithName(const std::string& name, const std::string& filter = {}, const std::string& versionRange = {}) {
+            waitIfAbleForEvents();
+            celix_service_filter_options_t opts{};
+            opts.serviceName = name.empty() ? nullptr : name.data();
+            opts.filter = filter.empty() ? nullptr : filter.data();
+            opts.versionRange = versionRange.empty() ? nullptr : versionRange.data();
+            return celix_bundleContext_findServiceWithOptions(cCtx.get(), &opts);
+        }
+#endif
 
         /**
          * @brief Finds all services matching the optional provided (LDAP) filter
@@ -206,10 +255,17 @@ namespace celix {
          * @param versionRange An optional version range.
          * @return A vector of service ids.
          */
+#if __cplusplus >= 201703L //C++17 or higher
         template<typename I>
         std::vector<long> findServices(std::string_view filter = {}, std::string_view versionRange = {}) {
             return findServicesWithName(celix::typeName<I>(), filter, versionRange);
         }
+#else
+        template<typename I>
+        std::vector<long> findServices(const std::string& filter = {}, const std::string& versionRange = {}) {
+            return findServicesWithName(celix::typeName<I>(), filter, versionRange);
+        }
+#endif
 
         /**
          * @brief Finds all service matching the provided service name and the optional (LDAP) filter
@@ -220,22 +276,21 @@ namespace celix {
          * @param versionRange An optional version range.
          * @return A vector of service ids.
          */
+#if __cplusplus >= 201703L //C++17 or higher
         std::vector<long> findServicesWithName(std::string_view name, std::string_view filter = {}, std::string_view versionRange = {}) {
-            waitIfAbleForEvents();
-            celix_service_filter_options_t opts{};
-            opts.serviceName = name.empty() ? nullptr : name.data();
-            opts.filter = filter.empty() ? nullptr : filter.data();
-            opts.versionRange = versionRange.empty() ? nullptr : versionRange.data();
-
-            std::vector<long> result{};
-            auto cList = celix_bundleContext_findServicesWithOptions(cCtx.get(), &opts);
-            for (int i = 0; i < celix_arrayList_size(cList); ++i) {
-                long svcId = celix_arrayList_getLong(cList, i);
-                result.push_back(svcId);
-            }
-            celix_arrayList_destroy(cList);
-            return result;
+            return findServicesWithNameInternal(
+                    name.empty() ? nullptr : name.data(),
+                    filter.empty() ? nullptr : filter.data(),
+                    versionRange.empty() ? nullptr : versionRange.data());
         }
+#else
+        std::vector<long> findServicesWithName(const std::string& name, const std::string& filter = {}, const std::string& versionRange = {}) {
+            return findServicesWithNameInternal(
+                    name.empty() ? nullptr : name.c_str(),
+                    filter.empty() ? nullptr : filter.c_str(),
+                    versionRange.empty() ? nullptr : versionRange.c_str());
+        }
+#endif
 
         /**
          * @brief Track services in the Celix framework using a fluent builder API.
@@ -257,10 +312,17 @@ namespace celix {
          * @param name The optional service name. If empty celix::typeName<I> will be used to defer the service name.
          * @return A ServiceTrackerBuilder object.
          */
+#if __cplusplus >= 201703L //C++17 or higher
         template<typename I>
         ServiceTrackerBuilder<I> trackServices(std::string_view name = {}) {
             return ServiceTrackerBuilder<I>{cCtx, celix::typeName<I>(name)};
         }
+#else
+        template<typename I>
+        ServiceTrackerBuilder<I> trackServices(const std::string& name = {}) {
+            return ServiceTrackerBuilder<I>{cCtx, celix::typeName<I>(name)};
+        }
+#endif
 
         /**
          * @brief Track services in the Celix framework using a fluent builder API.
@@ -272,6 +334,8 @@ namespace celix {
             return ServiceTrackerBuilder<void>{cCtx, {}};
         }
 
+        //TODO UseBundleBuilder useBundles()
+
         /**
          * @brief Track bundles in the Celix framework using a fluent builder API.
          *
@@ -310,10 +374,17 @@ namespace celix {
          * @param name The optional service name. If empty celix::typeName<I> will be used to defer the service name.
          * @return A MetaTrackerBuilder object.
          */
+#if __cplusplus >= 201703L //C++17 or higher
         template<typename I>
         MetaTrackerBuilder trackServiceTrackers(std::string_view name = {}) {
             return MetaTrackerBuilder(cCtx, celix::typeName<I>(name));
         }
+#else
+        template<typename I>
+        MetaTrackerBuilder trackServiceTrackers(const std::string& name = {}) {
+            return MetaTrackerBuilder(cCtx, celix::typeName<I>(name));
+        }
+#endif
 
         /**
          * @brief Track service trackers in the Celix framework using a fluent builder API.
@@ -324,6 +395,14 @@ namespace celix {
             return MetaTrackerBuilder(cCtx, {});
         }
 
+        /**
+         * @brief Schedule a callback to be called after the given initial delay and/or interval using a fluent 
+         * builder API.
+         */
+        ScheduledEventBuilder scheduledEvent() {
+            return ScheduledEventBuilder{cCtx};
+        }
+
         /**
          * @brief Install and optional start a bundle.
          *
@@ -333,9 +412,15 @@ namespace celix {
          * @param autoStart If the bundle should also be started.
          * @return the bundleId (>= 0) or < 0 if the bundle could not be installed and possibly started.
          */
+#if __cplusplus >= 201703L //C++17 or higher
         long installBundle(std::string_view bndLocation, bool autoStart = true) {
             return celix_bundleContext_installBundle(cCtx.get(), bndLocation.data(), autoStart);
         }
+#else
+        long installBundle(const std::string& bndLocation, bool autoStart = true) {
+            return celix_bundleContext_installBundle(cCtx.get(), bndLocation.c_str(), autoStart);
+        }
+#endif
 
         /**
          * @brief Uninstall the bundle with the provided bundle id.
@@ -355,8 +440,13 @@ namespace celix {
          *
          * Will silently ignore bundle ids < 0.
          *
+         * If this function is called on the Celix event thread, the actual starting of the bundle will be done async
+         * and on a separate thread.
+         * If this function is called from a different thread than the Celix event thread, then the function will
+         * return after the bundle start is completed.
+         *
          * @param bndId The bundle id to start.
-         * @return true if the bundle is found & correctly started. False if not.
+         * @return true if the bundle is found & correctly started or if the bundle is started async. False if not.
          */
         bool startBundle(long bndId) {
             return celix_bundleContext_startBundle(cCtx.get(), bndId);
@@ -367,20 +457,56 @@ namespace celix {
          *
          * Will silently ignore bundle ids < 0.
          *
+         * If this function is called on the Celix event thread, the actual stopping of the bundle will be done async
+         * and on a separate thread.
+         * If this function is called from a different thread than the Celix event thread, then the function will
+         * return after the bundle stop is completed.
+         *
          * @param bndId The bundle id to stop.
-         * @return true if the bundle is found & correctly stop. False if not.
+         * @return true if the bundle is found & correctly stop or if the bundle is stopped async. False if not.
          */
         bool stopBundle(long bndId) {
             return celix_bundleContext_stopBundle(cCtx.get(), bndId);
         }
 
+        /**
+         * @brief Update the bundle with the provided bundle id async.
+         *
+         * This will do the following:
+         *  - Stop the bundle (if needed);
+         *  - Update the bundle revision if a newer bundle zip if found;
+         *  - Start the bundle, if it was started.
+         *
+         * Will silently ignore bundle ids < 0.
+         *
+         * If this function is called on the Celix event thread, the actual updating of the bundle will be done async
+         * and on a separate thread.
+         * If this function is called from a different thread than the Celix event thread, then the function will
+         * return after the bundle update is completed.
+         *
+         * @param ctx The bundle context
+         * @param bndId The bundle id to start.
+         * @param updatedBundleUrl The optional updated bundle url to the bundle zip file. If empty, the existing
+         *                         bundle url from the bundle cache will be used.
+         * @return true if the bundle is found & correctly started or if the bundle is updated async. False if not.
+         */
+#if __cplusplus >= 201703L //C++17 or higher
+        bool updateBundle(long bndId, std::string_view updatedBundleUrl = {}) {
+            return celix_bundleContext_updateBundle(cCtx.get(), bndId, updatedBundleUrl.empty() ? nullptr : updatedBundleUrl.data());
+        }
+#else
+        bool updateBundle(long bndId, const std::string& updatedBundleUrl = {}) {
+            return celix_bundleContext_updateBundle(cCtx.get(), bndId, updatedBundleUrl.empty() ? nullptr : updatedBundleUrl.data());
+        }
+#endif
+
         /**
          * @brief List the installed and started bundle ids.
          * The bundle ids does not include the framework bundle (bundle id CELIX_FRAMEWORK_BUNDLE_ID).
          *
          * @return A vector with bundle ids.
          */
-        [[nodiscard]] std::vector<long> listBundleIds() const {
+        std::vector<long> listBundleIds() const {
             return listBundlesInternal(true);
         }
 
@@ -390,7 +516,7 @@ namespace celix {
          *
          * @return A vector with bundle ids.
          */
-        [[nodiscard]] std::vector<long> listInstalledBundleIds() {
+        std::vector<long> listInstalledBundleIds() {
             return listBundlesInternal(false);
         }
 
@@ -405,9 +531,15 @@ namespace celix {
          * @param defaultVal The default value to use if the property is not found.
          * @return The config property value for the provided key or the provided defaultValue is the name is not found.
          */
-        [[nodiscard]] std::string getConfigProperty(std::string_view name, std::string_view defaultValue) const {
+#if __cplusplus >= 201703L //C++17 or higher
+        std::string getConfigProperty(std::string_view name, std::string_view defaultValue) const {
             return std::string{celix_bundleContext_getProperty(cCtx.get(), name.data(), defaultValue.data())};
         }
+#else
+        std::string getConfigProperty(const std::string& name, const std::string& defaultValue) const {
+            return std::string{celix_bundleContext_getProperty(cCtx.get(), name.c_str(), defaultValue.c_str())};
+        }
+#endif
 
         /**
          * @brief Gets the config property for the provided name and returns it as a long.
@@ -421,9 +553,15 @@ namespace celix {
          * @return The config property value (as long) for the provided key or the provided defaultValue is the name
          * is not found or not a valid long.
          */
-        [[nodiscard]] long getConfigPropertyAsLong(std::string_view name, long defaultValue) const {
+#if __cplusplus >= 201703L //C++17 or higher
+        long getConfigPropertyAsLong(std::string_view name, long defaultValue) const {
             return celix_bundleContext_getPropertyAsLong(cCtx.get(), name.data(), defaultValue);
         }
+#else
+        long getConfigPropertyAsLong(const std::string& name, long defaultValue) const {
+            return celix_bundleContext_getPropertyAsLong(cCtx.get(), name.c_str(), defaultValue);
+        }
+#endif
 
         /**
          * @brief Gets the config property for the provided name and returns it as a double.
@@ -437,9 +575,15 @@ namespace celix {
          * @return The config property value (as double) for the provided key or the provided defaultValue is the name
          * is not found or not a valid double.
          */
-        [[nodiscard]] double getConfigPropertyAsDouble(std::string_view name, double defaultValue) const {
+#if __cplusplus >= 201703L //C++17 or higher
+        double getConfigPropertyAsDouble(std::string_view name, double defaultValue) const {
             return celix_bundleContext_getPropertyAsDouble(cCtx.get(), name.data(), defaultValue);
         }
+#else
+        double getConfigPropertyAsDouble(const std::string& name, double defaultValue) const {
+            return celix_bundleContext_getPropertyAsDouble(cCtx.get(), name.c_str(), defaultValue);
+        }
+#endif
 
         /**
          * @brief Gets the config property for the provided name and returns it as a bool.
@@ -455,28 +599,34 @@ namespace celix {
          * @return The config property value (as boolean) for the provided key or the provided defaultValue is the name
          * is not found or not a valid boolean.
          */
-        [[nodiscard]] long getConfigPropertyAsBool(std::string_view name, bool defaultValue) const {
+#if __cplusplus >= 201703L //C++17 or higher
+        long getConfigPropertyAsBool(std::string_view name, bool defaultValue) const {
             return celix_bundleContext_getPropertyAsBool(cCtx.get(), name.data(), defaultValue);
         }
+#else
+        long getConfigPropertyAsBool(const std::string& name, bool defaultValue) const {
+            return celix_bundleContext_getPropertyAsBool(cCtx.get(), name.c_str(), defaultValue);
+        }
+#endif
 
         /**
          * @brief Get the bundle of this bundle context.
          */
-        [[nodiscard]] const Bundle& getBundle() const {
+        const Bundle& getBundle() const {
             return bnd;
         }
 
         /**
          * @brief Get the bundle id for the bundle of this bundle context
          */
-        [[nodiscard]] long getBundleId() const {
+        long getBundleId() const {
              return bnd.getId();
          }
 
         /**
          * @brief Get the Celix framework for this bundle context.
          */
-        [[nodiscard]] std::shared_ptr<Framework> getFramework() const {
+        std::shared_ptr<Framework> getFramework() const {
             auto* cFw = celix_bundleContext_getFramework(cCtx.get());
             auto fwCtx = std::make_shared<celix::BundleContext>(celix_framework_getFrameworkContext(cFw));
             return std::make_shared<Framework>(fwCtx, cFw);
@@ -485,26 +635,26 @@ namespace celix {
         /**
          * @brief Get the Celix dependency manager for this bundle context
          */
-        [[nodiscard]] std::shared_ptr<dm::DependencyManager> getDependencyManager() const {
+        std::shared_ptr<dm::DependencyManager> getDependencyManager() const {
             return dm;
         }
 
         /**
          * @brief Get the C bundle context.
          *
-         * @warning Try not the depend on the C API from a C++ bundle. If features are missing these should be added to
-         * the C++ API.
+         * @warning Try not the depend on the C API from a C++ BundleContext.
+         * If features are missing these should be added to the C++ API.
          */
-        [[nodiscard]] celix_bundle_context_t* getCBundleContext() const {
+        celix_bundle_context_t* getCBundleContext() const {
             return cCtx.get();
         }
 
         /**
-         * @brief Logs a message to the Celix framework logger using the TRACE log level.
+         * @brief Log a message to the Celix framework logger using the TRACE log level.
          *
          * @note Only supports printf style call (so use c_str() instead of std::string)
          */
-        void logTrace(const char* format...) {
+        void logTrace(const char* format...) __attribute__((format(printf,2,3))) {
             va_list args;
             va_start(args, format);
             celix_bundleContext_vlog(cCtx.get(), CELIX_LOG_LEVEL_TRACE, format, args);
@@ -512,11 +662,11 @@ namespace celix {
         }
 
         /**
-         * @brief Logs a message to the Celix framework logger using the DEBUG log level.
+         * @brief Log a message to the Celix framework logger using the DEBUG log level.
          *
          * @note Only supports printf style call (so use c_str() instead of std::string)
          */
-        void logDebug(const char* format...) {
+        void logDebug(const char* format...) __attribute__((format(printf,2,3))) {
             va_list args;
             va_start(args, format);
             celix_bundleContext_vlog(cCtx.get(), CELIX_LOG_LEVEL_DEBUG, format, args);
@@ -524,11 +674,11 @@ namespace celix {
         }
 
         /**
-         * @brief Logs a message to the Celix framework logger using the INFO log level.
+         * @brief Log a message to the Celix framework logger using the INFO log level.
          *
          * @note Only supports printf style call (so use c_str() instead of std::string)
          */
-        void logInfo(const char* format...) {
+        void logInfo(const char* format...) __attribute__((format(printf,2,3))) {
             va_list args;
             va_start(args, format);
             celix_bundleContext_vlog(cCtx.get(), CELIX_LOG_LEVEL_INFO, format, args);
@@ -536,11 +686,11 @@ namespace celix {
         }
 
         /**
-         * @brief Logs a message to the Celix framework logger using the WARNING log level.
+         * @brief Log a message to the Celix framework logger using the WARNING log level.
          *
          * @note Only supports printf style call (so use c_str() instead of std::string)
          */
-        void logWarn(const char* format...) {
+        void logWarn(const char* format...) __attribute__((format(printf,2,3))) {
             va_list args;
             va_start(args, format);
             celix_bundleContext_vlog(cCtx.get(), CELIX_LOG_LEVEL_WARNING, format, args);
@@ -548,11 +698,11 @@ namespace celix {
         }
 
         /**
-         * @brief Logs a message to the Celix framework logger using the ERROR log level.
+         * @brief Log a message to the Celix framework logger using the ERROR log level.
          *
          * @note Only supports printf style call (so use c_str() instead of std::string)
          */
-        void logError(const char* format...) {
+        void logError(const char* format...) __attribute__((format(printf,2,3))) {
             va_list args;
             va_start(args, format);
             celix_bundleContext_vlog(cCtx.get(), CELIX_LOG_LEVEL_ERROR, format, args);
@@ -560,17 +710,25 @@ namespace celix {
         }
 
         /**
-         * @brief Logs a message to the Celix framework logger using the FATAL log level.
+         * @brief Log a message to the Celix framework logger using the FATAL log level.
          *
          * @note Only supports printf style call (so use c_str() instead of std::string)
          */
-        void logFatal(const char* format...) {
+        void logFatal(const char* format...) __attribute__((format(printf,2,3))) {
             va_list args;
             va_start(args, format);
             celix_bundleContext_vlog(cCtx.get(), CELIX_LOG_LEVEL_FATAL, format, args);
             va_end(args);
         }
 
+        /**
+         * @brief Log celix per-thread error messages to the Celix framework logger using the provided log level.
+         * Silently ignores log level CELIX_LOG_LEVEL_DISABLED.
+         */
+        void logTssErrors(celix_log_level_e level) {
+            celix_bundleContext_logTssErrors(cCtx.get(), level);
+        }
+
         /**
          * @brief Wait until all Celix events for this bundle are completed.
          */
@@ -606,7 +764,7 @@ namespace celix {
             }
          }
     private:
-        [[nodiscard]] std::vector<long> listBundlesInternal(bool activeOnly) const {
+        std::vector<long> listBundlesInternal(bool activeOnly) const {
             std::vector<long> result{};
             auto* ids = activeOnly ?
                         celix_bundleContext_listBundles(getCBundleContext()) :
@@ -619,6 +777,23 @@ namespace celix {
             return result;
         }
 
+        std::vector<long> findServicesWithNameInternal(const char* name, const char* filter, const char* versionRange) {
+            waitIfAbleForEvents();
+            celix_service_filter_options_t opts{};
+            opts.serviceName = name;
+            opts.filter = filter;
+            opts.versionRange = versionRange;
+
+            std::vector<long> result{};
+            auto cList = celix_bundleContext_findServicesWithOptions(cCtx.get(), &opts);
+            for (int i = 0; i < celix_arrayList_size(cList); ++i) {
+                long svcId = celix_arrayList_getLong(cList, i);
+                result.push_back(svcId);
+            }
+            celix_arrayList_destroy(cList);
+            return result;
+        }
+
         const std::shared_ptr<celix_bundle_context_t> cCtx;
         const std::shared_ptr<celix::dm::DependencyManager> dm;
         const Bundle bnd;
