@@ -41,7 +41,7 @@
 #include "service_tracker_private.h"
 #include "service_reference_private.h"
 #include "celix_array_list.h"
-#include "celix_libloader.h"
+#include "celix_convert_utils.h"
 
 static celix_status_t bundleContext_bundleChanged(void *handle, bundle_event_t *event);
 static void bundleContext_cleanupBundleTrackers(bundle_context_t *ct);
@@ -111,115 +111,78 @@ celix_status_t bundleContext_destroy(bundle_context_pt context) {
 	return status;
 }
 
-void celix_bundleContext_cleanup(celix_bundle_context_t *ctx) {
-    //NOTE not perfect, because stopping of registrations/tracker when the activator is destroyed can lead to segfault.
-    //but at least we can try to warn the bundle implementer that some cleanup is missing.
-    bundleContext_cleanupBundleTrackers(ctx);
-    bundleContext_cleanupServiceTrackers(ctx);
-    bundleContext_cleanupServiceTrackerTrackers(ctx);
-    bundleContext_cleanupServiceRegistration(ctx);
+void celix_bundleContext_cleanup(celix_bundle_context_t* ctx) {
+        fw_log(ctx->framework->logger,
+               CELIX_LOG_LEVEL_TRACE,
+               "Cleaning up bundle context `%s` (id=%li)",
+               celix_bundle_getSymbolicName(ctx->bundle),
+               celix_bundle_getId(ctx->bundle));
+
+        celix_framework_cleanupScheduledEvents(ctx->framework, celix_bundle_getId(ctx->bundle));
+        // NOTE not perfect, because stopping of registrations/tracker when the activator is destroyed can lead to
+        // segfault. but at least we can try to warn the bundle implementer that some cleanup is missing.
+        bundleContext_cleanupBundleTrackers(ctx);
+        bundleContext_cleanupServiceTrackers(ctx);
+        bundleContext_cleanupServiceTrackerTrackers(ctx);
+        bundleContext_cleanupServiceRegistration(ctx);
 }
 
 celix_status_t bundleContext_getBundle(bundle_context_pt context, bundle_pt *out) {
-	celix_status_t status = CELIX_SUCCESS;
-    celix_bundle_t *bnd = celix_bundleContext_getBundle(context);
-    if (context == NULL) {
-        status = CELIX_ILLEGAL_ARGUMENT;
+    if (context == NULL || out == NULL) {
+        return CELIX_ILLEGAL_ARGUMENT;
     }
-    if (out != NULL) {
-        *out = bnd;
-    }
-
-    framework_logIfError(context->framework->logger, status, NULL, "Failed to get bundle");
-
-	return status;
+    *out = celix_bundleContext_getBundle(context);
+    return CELIX_SUCCESS;
 }
 
 celix_status_t bundleContext_getFramework(bundle_context_pt context, framework_pt *framework) {
-	celix_status_t status = CELIX_SUCCESS;
-
-	if (context == NULL) {
-		status = CELIX_ILLEGAL_ARGUMENT;
-	} else {
-		*framework = context->framework;
-	}
-
-	framework_logIfError(context->framework->logger, status, NULL, "Failed to get framework");
-
-	return status;
+    if (context == NULL || framework == NULL) {
+        return CELIX_ILLEGAL_ARGUMENT;
+    }
+    *framework = context->framework;
+    return CELIX_SUCCESS;
 }
 
 celix_status_t bundleContext_installBundle(bundle_context_pt context, const char * location, bundle_pt *bundle) {
-	return bundleContext_installBundle2(context, location, NULL, bundle);
+    return bundleContext_installBundle2(context, location, NULL, bundle);
 }
 
 celix_status_t bundleContext_installBundle2(bundle_context_pt context, const char *location, const char *inputFile, bundle_pt *bundle) {
-	celix_status_t status = CELIX_SUCCESS;
-	bundle_pt b = NULL;
-
-	if (context != NULL) {
-		if (fw_installBundle(context->framework, &b, location, inputFile) != CELIX_SUCCESS) {
-            status = CELIX_FRAMEWORK_EXCEPTION;
-		} else {
-			*bundle = b;
-		}
-	} else {
-        status = CELIX_ILLEGAL_ARGUMENT;
-	}
-
-	framework_logIfError(context->framework->logger, status, NULL, "Failed to install bundle");
-
-	return status;
+    celix_status_t status = CELIX_SUCCESS;
+    long id = -1L;
+    if (context == NULL || location == NULL || bundle == NULL) {
+        return CELIX_ILLEGAL_ARGUMENT;
+    }
+    status = celix_framework_installBundleInternal(context->framework, location, &id);
+    if (status == CELIX_SUCCESS) {
+        *bundle = framework_getBundleById(context->framework, id);
+    }
+    return status;
 }
 
 celix_status_t bundleContext_registerService(bundle_context_pt context, const char * serviceName, const void * svcObj,
         properties_pt properties, service_registration_pt *service_registration) {
-	service_registration_pt registration = NULL;
-	celix_status_t status = CELIX_SUCCESS;
-
-	if (context != NULL) {
-	    long bndId = celix_bundle_getId(context->bundle);
-	    fw_registerService(context->framework, &registration, bndId, serviceName, svcObj, properties);
-	    *service_registration = registration;
-	} else {
-	    status = CELIX_ILLEGAL_ARGUMENT;
-	}
-
-	framework_logIfError(context->framework->logger, status, NULL, "Failed to register service. serviceName '%s'", serviceName);
-
-	return status;
+    if (context == NULL || service_registration == NULL) {
+        return CELIX_ILLEGAL_ARGUMENT;
+    }
+    long bndId = celix_bundle_getId(context->bundle);
+    return fw_registerService(context->framework, service_registration, bndId, serviceName, svcObj, properties);
 }
 
 celix_status_t bundleContext_registerServiceFactory(bundle_context_pt context, const char * serviceName, service_factory_pt factory,
         properties_pt properties, service_registration_pt *service_registration) {
-    service_registration_pt registration = NULL;
-    celix_status_t status = CELIX_SUCCESS;
-
-    if (context != NULL && *service_registration == NULL) {
-        long bndId = celix_bundle_getId(context->bundle);
-        fw_registerServiceFactory(context->framework, &registration, bndId, serviceName, factory, properties);
-        *service_registration = registration;
-    } else {
-        status = CELIX_ILLEGAL_ARGUMENT;
+    if (context == NULL || service_registration == NULL) {
+        return CELIX_ILLEGAL_ARGUMENT;
     }
-
-    framework_logIfError(context->framework->logger, status, NULL, "Failed to register service factory");
-
-    return status;
+    long bndId = celix_bundle_getId(context->bundle);
+    return fw_registerServiceFactory(context->framework, service_registration, bndId, serviceName, factory, properties);
 }
 
 celix_status_t bundleContext_getServiceReferences(bundle_context_pt context, const char * serviceName, const char * filter, array_list_pt *service_references) {
-    celix_status_t status = CELIX_SUCCESS;
-
-    if (context != NULL && *service_references == NULL) {
-        fw_getServiceReferences(context->framework, service_references, context->bundle, serviceName, filter);
-    } else {
-        status = CELIX_ILLEGAL_ARGUMENT;
+    if (context == NULL || service_references == NULL) {
+        return CELIX_ILLEGAL_ARGUMENT;
     }
-
-    framework_logIfError(context->framework->logger, status, NULL, "Failed to get service references");
-
-	return status;
+    return fw_getServiceReferences(context->framework, service_references, context->bundle, serviceName, filter);
 }
 
 celix_status_t bundleContext_getServiceReference(bundle_context_pt context, const char * serviceName, service_reference_pt *service_reference) {
@@ -257,173 +220,100 @@ static bool bundleContext_IsServiceReferenceValid(bundle_context_pt context, ser
     return context->bundle == refBundle;
 }
 
-FRAMEWORK_EXPORT celix_status_t bundleContext_retainServiceReference(bundle_context_pt context, service_reference_pt ref) {
-    celix_status_t status = CELIX_SUCCESS;
-
-    if (context != NULL && ref != NULL && bundleContext_IsServiceReferenceValid(context, ref)) {
-        serviceReference_retain(ref);
-    } else {
-        status = CELIX_ILLEGAL_ARGUMENT;
+celix_status_t bundleContext_retainServiceReference(bundle_context_pt context, service_reference_pt ref) {
+    if (context == NULL || ref == NULL || !bundleContext_IsServiceReferenceValid(context, ref)) {
+        return CELIX_ILLEGAL_ARGUMENT;
     }
-
-    framework_logIfError(context->framework->logger, status, NULL, "Failed to retain service references");
-
-    return status;
+    return serviceReference_retain(ref);
 }
 
 celix_status_t bundleContext_ungetServiceReference(bundle_context_pt context, service_reference_pt reference) {
-    celix_status_t status = CELIX_SUCCESS;
-
-    if (context != NULL && reference != NULL && bundleContext_IsServiceReferenceValid(context, reference)) {
-        serviceReference_release(reference, NULL);
-    } else {
-        status = CELIX_ILLEGAL_ARGUMENT;
+    if (context == NULL || reference == NULL || !bundleContext_IsServiceReferenceValid(context, reference)) {
+        return CELIX_ILLEGAL_ARGUMENT;
     }
-
-    framework_logIfError(context->framework->logger, status, NULL, "Failed to unget service_reference");
-
-    return status;
+    return serviceReference_release(reference, NULL);
 }
 
 celix_status_t bundleContext_getService(bundle_context_pt context, service_reference_pt reference, void** service_instance) {
-    celix_status_t status = CELIX_SUCCESS;
-    if (context != NULL && reference != NULL && service_instance != NULL && bundleContext_IsServiceReferenceValid(context, reference)) {
-        /*NOTE argument service_instance should be considered a 'const void**'.
-        To ensure backwards compatibility a cast is made instead.
-        */
-        status = serviceReference_getService(reference, (const void**) service_instance);
-    } else {
-        status = CELIX_ILLEGAL_ARGUMENT;
+    if (context == NULL || reference == NULL || service_instance == NULL || !bundleContext_IsServiceReferenceValid(context, reference)) {
+        return CELIX_ILLEGAL_ARGUMENT;
     }
-    framework_logIfError(context->framework->logger, status, NULL, "Failed to get service");
-    return status;
+    /*NOTE argument service_instance should be considered a 'const void**'.
+    To ensure backwards compatibility a cast is made instead.
+    */
+    return serviceReference_getService(reference, (const void**) service_instance);
 }
 
 celix_status_t bundleContext_ungetService(bundle_context_pt context, service_reference_pt reference, bool *result) {
-    celix_status_t status = CELIX_SUCCESS;
-    if (context != NULL && reference != NULL && bundleContext_IsServiceReferenceValid(context, reference)) {
-        status = serviceReference_ungetService(reference, result);
-    } else {
-        status = CELIX_ILLEGAL_ARGUMENT;
+    if (context == NULL || reference == NULL || !bundleContext_IsServiceReferenceValid(context, reference)) {
+        return CELIX_ILLEGAL_ARGUMENT;
     }
-    framework_logIfError(context->framework->logger, status, NULL, "Failed to unget service");
-
-    return status;
+    return serviceReference_ungetService(reference, result);
 }
 
 celix_status_t bundleContext_getBundles(bundle_context_pt context, array_list_pt *bundles) {
-	celix_status_t status = CELIX_SUCCESS;
-
-	if (context == NULL || *bundles != NULL) {
-		status = CELIX_ILLEGAL_ARGUMENT;
-	} else {
-		*bundles = framework_getBundles(context->framework);
-	}
-
-	framework_logIfError(context->framework->logger, status, NULL, "Failed to get bundles");
-
-	return status;
+    if (context == NULL || bundles == NULL) {
+        return CELIX_ILLEGAL_ARGUMENT;
+    }
+    *bundles = framework_getBundles(context->framework);
+    return CELIX_SUCCESS;
 }
 
 celix_status_t bundleContext_getBundleById(bundle_context_pt context, long id, bundle_pt *bundle) {
+    if (context == NULL || bundle == NULL) {
+        return CELIX_ILLEGAL_ARGUMENT;
+    }
     celix_status_t status = CELIX_SUCCESS;
-
-    if (context == NULL || *bundle != NULL) {
-        status = CELIX_ILLEGAL_ARGUMENT;
-    } else {
-        *bundle = framework_getBundleById(context->framework, id);
-        if (*bundle == NULL) {
-            status = CELIX_BUNDLE_EXCEPTION;
-        }
+    *bundle = framework_getBundleById(context->framework, id);
+    if (*bundle == NULL) {
+        status = CELIX_BUNDLE_EXCEPTION;
     }
-
     framework_logIfError(context->framework->logger, status, NULL, "Failed to get bundle [id=%ld]", id);
-
-	return status;
+    return status;
 }
 
 celix_status_t bundleContext_addServiceListener(bundle_context_pt context, celix_service_listener_t *listener, const char* filter) {
-    celix_status_t status = CELIX_SUCCESS;
-
-    if (context != NULL && listener != NULL) {
-        fw_addServiceListener(context->framework, context->bundle, listener, filter);
-    } else {
-        status = CELIX_ILLEGAL_ARGUMENT;
+    if (context == NULL || listener == NULL) {
+        return CELIX_ILLEGAL_ARGUMENT;
     }
-
-    framework_logIfError(context->framework->logger, status, NULL, "Failed to add service listener");
-
-    return status;
+    fw_addServiceListener(context->framework, context->bundle, listener, filter);
+    return CELIX_SUCCESS;
 }
 
 celix_status_t bundleContext_removeServiceListener(bundle_context_pt context, celix_service_listener_t *listener) {
-    celix_status_t status = CELIX_SUCCESS;
-
-    if (context != NULL && listener != NULL) {
-        fw_removeServiceListener(context->framework, context->bundle, listener);
-    } else {
-        status = CELIX_ILLEGAL_ARGUMENT;
+    if (context == NULL || listener == NULL) {
+        return CELIX_ILLEGAL_ARGUMENT;
     }
-
-    framework_logIfError(context->framework->logger, status, NULL, "Failed to remove service listener");
-
-    return status;
+    fw_removeServiceListener(context->framework, context->bundle, listener);
+    return CELIX_SUCCESS;
 }
 
 celix_status_t bundleContext_addBundleListener(bundle_context_pt context, bundle_listener_pt listener) {
-    celix_status_t status = CELIX_SUCCESS;
-
-    if (context != NULL && listener != NULL) {
-        fw_addBundleListener(context->framework, context->bundle, listener);
-    } else {
-        status = CELIX_ILLEGAL_ARGUMENT;
+    if (context == NULL || listener == NULL) {
+        return CELIX_ILLEGAL_ARGUMENT;
     }
-
-    framework_logIfError(context->framework->logger, status, NULL, "Failed to add bundle listener");
-
-    return status;
+    return fw_addBundleListener(context->framework, context->bundle, listener);
 }
 
 celix_status_t bundleContext_removeBundleListener(bundle_context_pt context, bundle_listener_pt listener) {
-    celix_status_t status = CELIX_SUCCESS;
-
-    if (context != NULL && listener != NULL) {
-        fw_removeBundleListener(context->framework, context->bundle, listener);
-    } else {
-        status = CELIX_ILLEGAL_ARGUMENT;
+    if (context == NULL || listener == NULL) {
+        return CELIX_ILLEGAL_ARGUMENT;
     }
-
-    framework_logIfError(context->framework->logger, status, NULL, "Failed to remove bundle listener");
-
-    return status;
+    return fw_removeBundleListener(context->framework, context->bundle, listener);
 }
 
 celix_status_t bundleContext_addFrameworkListener(bundle_context_pt context, framework_listener_pt listener) {
-    celix_status_t status = CELIX_SUCCESS;
-
-    if (context != NULL && listener != NULL) {
-        fw_addFrameworkListener(context->framework, context->bundle, listener);
-    } else {
-        status = CELIX_ILLEGAL_ARGUMENT;
+    if (context == NULL || listener == NULL) {
+        return CELIX_ILLEGAL_ARGUMENT;
     }
-
-    framework_logIfError(context->framework->logger, status, NULL, "Failed to add framework listener");
-
-    return status;
+    return fw_addFrameworkListener(context->framework, context->bundle, listener);
 }
 
 celix_status_t bundleContext_removeFrameworkListener(bundle_context_pt context, framework_listener_pt listener) {
-    celix_status_t status = CELIX_SUCCESS;
-
-    if (context != NULL && listener != NULL) {
-        fw_removeFrameworkListener(context->framework, context->bundle, listener);
-    } else {
-        status = CELIX_ILLEGAL_ARGUMENT;
+    if (context == NULL || listener == NULL) {
+        return CELIX_ILLEGAL_ARGUMENT;
     }
-
-    framework_logIfError(context->framework->logger, status, NULL, "Failed to remove framework listener");
-
-    return status;
+    return fw_removeFrameworkListener(context->framework, context->bundle, listener);
 }
 
 celix_status_t bundleContext_getProperty(bundle_context_pt context, const char *name, const char** value) {
@@ -632,10 +522,7 @@ celix_dependency_manager_t* celix_bundleContext_getDependencyManager(bundle_cont
 static celix_status_t bundleContext_bundleChanged(void *listenerSvc, bundle_event_t *event) {
     celix_status_t status = CELIX_SUCCESS;
     bundle_listener_t *listener = listenerSvc;
-    celix_bundle_context_bundle_tracker_entry_t *tracker = NULL;
-    if (listener != NULL) {
-        tracker = listener->handle;
-    }
+    celix_bundle_context_bundle_tracker_entry_t *tracker = listener->handle;
 
     bool handleEvent = true;
     long bndId = celix_bundle_getId(event->bnd);
@@ -643,7 +530,7 @@ static celix_status_t bundleContext_bundleChanged(void *listenerSvc, bundle_even
         handleEvent = tracker->opts.includeFrameworkBundle;
     }
 
-    if (tracker != NULL && handleEvent) {
+    if (handleEvent) {
         void *callbackHandle = tracker->opts.callbackHandle;
 
         if (event->type == OSGI_FRAMEWORK_BUNDLE_EVENT_INSTALLED && tracker->opts.onInstalled != NULL) {
@@ -756,11 +643,11 @@ long celix_bundleContext_trackBundlesAsync(
     return celix_bundleContext_trackBundlesWithOptionsAsync(ctx, &opts);
 }
 
-void celix_bundleContext_useBundles(
+size_t celix_bundleContext_useBundles(
         bundle_context_t *ctx,
         void *callbackHandle,
         void (*use)(void *handle, const bundle_t *bundle)) {
-    celix_framework_useBundles(ctx->framework, false, callbackHandle, use);
+    return celix_framework_useBundles(ctx->framework, false, callbackHandle, use);
 }
 
 bool celix_bundleContext_useBundle(
@@ -768,7 +655,7 @@ bool celix_bundleContext_useBundle(
         long bundleId,
         void *callbackHandle,
         void (*use)(void *handle, const bundle_t *bundle)) {
-    return celix_framework_useBundle(ctx->framework, true, bundleId, callbackHandle, use);
+    return celix_framework_useBundle(ctx->framework, false, bundleId, callbackHandle, use);
 }
 
 static void bundleContext_cleanupBundleTrackers(bundle_context_t *ctx) {
@@ -1062,10 +949,8 @@ static void celix_bundleContext_waitForTrackerInternal(celix_bundle_context_t* c
     }
 
     if (found) {
-        if (svcId >= 0 && waitForStart) {
+        if (svcId >= 0) {
             celix_framework_waitForAsyncRegistration(ctx->framework, svcId);
-        } else if (svcId >= 0) {
-            celix_framework_waitForAsyncUnregistration(ctx->framework, svcId);
         } else {
             celix_framework_waitForGenericEvent(ctx->framework, eventId);
         }
@@ -1101,17 +986,24 @@ bool celix_bundleContext_isBundleActive(celix_bundle_context_t *ctx, long bndId)
     return celix_framework_isBundleActive(ctx->framework, bndId);
 }
 
-bool celix_bundleContext_startBundle(celix_bundle_context_t *ctx, long bundleId) {
-    return celix_framework_startBundle(ctx->framework, bundleId);
+bool celix_bundleContext_startBundle(celix_bundle_context_t *ctx, long bndId) {
+    return celix_framework_startBundle(ctx->framework, bndId);
 }
 
-
-bool celix_bundleContext_stopBundle(celix_bundle_context_t *ctx, long bundleId) {
-    return celix_framework_stopBundle(ctx->framework, bundleId);
+bool celix_bundleContext_updateBundle(celix_bundle_context_t *ctx, long bndId, const char* updatedBundleUrl) {
+    return celix_framework_updateBundle(ctx->framework, bndId, updatedBundleUrl);
 }
 
-bool celix_bundleContext_uninstallBundle(bundle_context_t *ctx, long bundleId) {
-    return celix_framework_uninstallBundle(ctx->framework, bundleId);
+bool celix_bundleContext_stopBundle(celix_bundle_context_t *ctx, long bndId) {
+    return celix_framework_stopBundle(ctx->framework, bndId);
+}
+
+bool celix_bundleContext_uninstallBundle(bundle_context_t *ctx, long bndId) {
+    return celix_framework_uninstallBundle(ctx->framework, bndId);
+}
+
+bool celix_bundleContext_unloadBundle(celix_bundle_context_t *ctx, long bndId) {
+    return celix_framework_unloadBundle(ctx->framework, bndId);
 }
 
 bool celix_bundleContext_useServiceWithId(
@@ -1211,7 +1103,7 @@ bool celix_bundleContext_useServiceWithOptions(
 
     if(opts->flags & CELIX_SERVICE_USE_DIRECT) {
         if(opts->flags & CELIX_SERVICE_USE_SOD) {
-            // check CelixBundleContextServicesTests.UseServiceOnDemandDirectlyWithAsyncRegisterTest to see what is "service on demand".
+            // check CelixBundleContextServicesTestSuite.UseServiceOnDemandDirectlyWithAsyncRegisterTest to see what is "service on demand".
             celix_framework_waitUntilNoPendingRegistration(ctx->framework);
         }
         called = celix_serviceTracker_useHighestRankingService(data.svcTracker, NULL, opts->waitTimeoutInSeconds, opts->callbackHandle, opts->use, opts->useWithProperties, opts->useWithOwner);
@@ -1268,7 +1160,7 @@ size_t celix_bundleContext_useServicesWithOptions(
 
     if (opts->flags & CELIX_SERVICE_USE_DIRECT) {
         if(opts->flags & CELIX_SERVICE_USE_SOD) {
-            // check CelixBundleContextServicesTests.UseServicesOnDemandDirectlyWithAsyncRegisterTest to see what is "service on demand".
+            // check CelixBundleContextServicesTestSuite.UseServicesOnDemandDirectlyWithAsyncRegisterTest to see what is "service on demand".
             celix_framework_waitUntilNoPendingRegistration(ctx->framework);
         }
         celix_bundleContext_useServicesWithOptions_2_UseServiceTracker(&data);
@@ -1343,25 +1235,20 @@ static void celix_bundleContext_createTrackerOnEventLoop(void *data) {
     assert(celix_framework_isCurrentThreadTheEventLoop(entry->ctx->framework));
     celixThreadMutex_lock(&entry->ctx->mutex);
     bool cancelled = entry->cancelled;
-    celixThreadMutex_unlock(&entry->ctx->mutex);
     if (cancelled) {
         fw_log(entry->ctx->framework->logger, CELIX_LOG_LEVEL_DEBUG, "Creating of service tracker was cancelled. trk id = %li, svc name tracked = %s", entry->trackerId, entry->opts.filter.serviceName);
+        celixThreadMutex_unlock(&entry->ctx->mutex);
+        return;
+    }
+    celix_service_tracker_t *tracker = celix_serviceTracker_createClosedWithOptions(entry->ctx, &entry->opts);
+    if (tracker == NULL) {
+        fw_log(entry->ctx->framework->logger, CELIX_LOG_LEVEL_ERROR, "Cannot create tracker for bnd %s (%li)", celix_bundle_getSymbolicName(entry->ctx->bundle), celix_bundle_getId(entry->ctx->bundle));
     } else {
-        celix_service_tracker_t *tracker = celix_serviceTracker_createWithOptions(entry->ctx, &entry->opts);
-        if (tracker != NULL) {
-            celixThreadMutex_lock(&entry->ctx->mutex);
-            // double-check is necessary to eliminate first-check-then-do race condition
-            if(!entry->cancelled) {
-                entry->tracker = tracker;
-            }
-            celixThreadMutex_unlock(&entry->ctx->mutex);
-            if(entry->tracker == NULL) {
-                assert(entry->cancelled);
-                celix_serviceTracker_destroy(tracker);
-            }
-        } else {
-            fw_log(entry->ctx->framework->logger, CELIX_LOG_LEVEL_ERROR, "Cannot create tracker for bnd %s (%li)", celix_bundle_getSymbolicName(entry->ctx->bundle), celix_bundle_getId(entry->ctx->bundle));
-        }
+        entry->tracker = tracker;
+    }
+    celixThreadMutex_unlock(&entry->ctx->mutex);
+    if (tracker) {
+        serviceTracker_open(tracker);
     }
 }
 
@@ -1504,12 +1391,13 @@ static celix_status_t bundleContext_callServicedTrackerTrackerCallback(void *han
             celix_bundle_t *bnd = NULL;
             bundleContext_getBundle(info->context, &bnd);
 
+            celix_autoptr(celix_filter_t) filter = celix_filter_create(info->filter);
             celix_service_tracker_info_t trkInfo;
             memset(&trkInfo, 0, sizeof(trkInfo));
             trkInfo.bundleId = celix_bundle_getId(bnd);
-            trkInfo.filter = celix_filter_create(info->filter);
-            trkInfo.serviceName = celix_filter_findAttribute(trkInfo.filter, OSGI_FRAMEWORK_OBJECTCLASS);
-            const char *filterSvcName = celix_filter_findAttribute(trkInfo.filter, OSGI_FRAMEWORK_OBJECTCLASS);
+            trkInfo.filter = filter;
+            trkInfo.serviceName = celix_filter_findAttribute(filter, OSGI_FRAMEWORK_OBJECTCLASS);
+            const char *filterSvcName = celix_filter_findAttribute(filter, OSGI_FRAMEWORK_OBJECTCLASS);
 
             bool match = entry->serviceName == NULL || (filterSvcName != NULL && strncmp(filterSvcName, entry->serviceName, 1024*1024) == 0);
 
@@ -1518,7 +1406,6 @@ static celix_status_t bundleContext_callServicedTrackerTrackerCallback(void *han
             } else if (!add && entry->remove != NULL && match) {
                 entry->remove(entry->callbackHandle, &trkInfo);
             }
-            celix_filter_destroy(trkInfo.filter);
         }
     }
     return CELIX_SUCCESS;
@@ -1542,7 +1429,7 @@ long celix_bundleContext_trackServiceTrackersInternal(
         void *doneData,
         void (*doneCallback)(void*)) {
     if (serviceName == NULL) {
-        fw_log(ctx->framework->logger, CELIX_LOG_LEVEL_DEBUG, "Note starting a meta tracker for all services", __FUNCTION__);
+        fw_log(ctx->framework->logger, CELIX_LOG_LEVEL_DEBUG, "Note starting a meta tracker for all services");
     }
 
     celix_bundle_context_service_tracker_tracker_entry_t *entry = calloc(1, sizeof(*entry));
@@ -1607,6 +1494,41 @@ void celix_bundleContext_waitForEvents(celix_bundle_context_t* ctx) {
     celix_framework_waitUntilNoEventsForBnd(ctx->framework, celix_bundle_getId(ctx->bundle));
 }
 
+long celix_bundleContext_scheduleEvent(celix_bundle_context_t* ctx,
+                                       const celix_scheduled_event_options_t* options) {
+    return celix_framework_scheduleEvent(ctx->framework,
+                                          celix_bundle_getId(ctx->bundle),
+                                          options->name,
+                                          options->initialDelayInSeconds,
+                                          options->intervalInSeconds,
+                                          options->callbackData,
+                                          options->callback,
+                                          options->removeCallbackData,
+                                          options->removeCallback);
+}
+
+celix_status_t celix_bundleContext_wakeupScheduledEvent(celix_bundle_context_t* ctx, long scheduledEventId) {
+    return celix_framework_wakeupScheduledEvent(ctx->framework, scheduledEventId);
+}
+
+celix_status_t celix_bundleContext_waitForScheduledEvent(celix_bundle_context_t* ctx,
+                                                         long scheduledEventId,
+                                                         double waitTimeInSeconds) {
+    return celix_framework_waitForScheduledEvent(ctx->framework, scheduledEventId, waitTimeInSeconds);
+}
+
+bool celix_bundleContext_removeScheduledEvent(celix_bundle_context_t* ctx, long scheduledEventId) {
+    return celix_framework_removeScheduledEvent(ctx->framework, false, true, scheduledEventId);
+}
+
+bool celix_bundleContext_removeScheduledEventAsync(celix_bundle_context_t* ctx, long scheduledEventId) {
+    return celix_framework_removeScheduledEvent(ctx->framework, true, true, scheduledEventId);
+}
+
+bool celix_bundleContext_tryRemoveScheduledEventAsync(celix_bundle_context_t* ctx, long scheduledEventId) {
+    return celix_framework_removeScheduledEvent(ctx->framework, true, false, scheduledEventId);
+}
+
 celix_bundle_t* celix_bundleContext_getBundle(const celix_bundle_context_t *ctx) {
     celix_bundle_t *bnd = NULL;
     if (ctx != NULL) {
@@ -1628,57 +1550,36 @@ celix_framework_t* celix_bundleContext_getFramework(const celix_bundle_context_t
     return fw;
 }
 
-const char* celix_bundleContext_getProperty(celix_bundle_context_t *ctx, const char *key, const char *defaultVal) {
-    const char *val = NULL;
+const char* celix_bundleContext_getProperty(celix_bundle_context_t *ctx, const char *key, const char* defaultValue) {
+    const char* val = defaultValue;
     if (ctx != NULL) {
-        fw_getProperty(ctx->framework, key, defaultVal, &val);
+        val = celix_framework_getConfigProperty(ctx->framework, key, defaultValue, NULL);
     }
     return val;
 }
 
 long celix_bundleContext_getPropertyAsLong(celix_bundle_context_t *ctx, const char *key, long defaultValue) {
-    long result = defaultValue;
-    const char *val = celix_bundleContext_getProperty(ctx, key, NULL);
-    if (val != NULL) {
-        char *enptr = NULL;
-        errno = 0;
-        long r = strtol(val, &enptr, 10);
-        if (enptr != val && errno == 0) {
-            result = r;
-        }
+    long val = defaultValue;
+    if (ctx != NULL) {
+        val = celix_framework_getConfigPropertyAsLong(ctx->framework, key, defaultValue, NULL);
     }
-    return result;
+    return val;
 }
 
 double celix_bundleContext_getPropertyAsDouble(celix_bundle_context_t *ctx, const char *key, double defaultValue) {
-    double result = defaultValue;
-    const char *val = celix_bundleContext_getProperty(ctx, key, NULL);
-    if (val != NULL) {
-        char *enptr = NULL;
-        errno = 0;
-        double r = strtod(val, &enptr);
-        if (enptr != val && errno == 0) {
-            result = r;
-        }
+    double val = defaultValue;
+    if (ctx != NULL) {
+        val = celix_framework_getConfigPropertyAsDouble(ctx->framework, key, defaultValue, NULL);
     }
-    return result;
+    return val;
 }
 
-
 bool celix_bundleContext_getPropertyAsBool(celix_bundle_context_t *ctx, const char *key, bool defaultValue) {
-    bool result = defaultValue;
-    const char *val = celix_bundleContext_getProperty(ctx, key, NULL);
-    if (val != NULL) {
-        char buf[32];
-        snprintf(buf, 32, "%s", val);
-        char *trimmed = utils_stringTrim(buf);
-        if (strncasecmp("true", trimmed, strlen("true")) == 0) {
-            result = true;
-        } else if (strncasecmp("false", trimmed, strlen("false")) == 0) {
-            result = false;
-        }
+    bool val = defaultValue;
+    if (ctx != NULL) {
+        val = celix_framework_getConfigPropertyAsBool(ctx->framework, key, defaultValue, NULL);
     }
-    return result;
+    return val;
 }
 
 static void celix_bundleContext_getBundleSymbolicNameCallback(void *data, const celix_bundle_t *bnd) {
@@ -1702,3 +1603,7 @@ void celix_bundleContext_log(const celix_bundle_context_t *ctx, celix_log_level_
 void celix_bundleContext_vlog(const celix_bundle_context_t *ctx, celix_log_level_e level, const char *format, va_list formatArgs) {
     celix_framework_vlog(ctx->framework->logger, level, NULL, NULL, 0, format, formatArgs);
 }
+
+void celix_bundleContext_logTssErrors(const celix_bundle_context_t *ctx, celix_log_level_e level) {
+    celix_framework_logTssErrors(ctx->framework->logger, level);
+}
\ No newline at end of file
