@@ -122,7 +122,7 @@ celix_status_t serviceTracker_createWithFilter(bundle_context_pt context, const
     celixThreadCondition_init(&tracker->condTracked, NULL);
     celixThreadCondition_init(&tracker->condUntracking, NULL);
     tracker->trackedServices = celix_arrayList_create();
-    tracker->untrackingServices = celix_arrayList_create();
+    tracker->untrackedServiceCount = 0;
 
     tracker->currentHighestServiceId = -1;
 
@@ -141,7 +141,6 @@ celix_status_t serviceTracker_destroy(service_tracker_pt tracker) {
     celixThreadCondition_destroy(&tracker->condTracked);
     celixThreadCondition_destroy(&tracker->condUntracking);
     celix_arrayList_destroy(tracker->trackedServices);
-    celix_arrayList_destroy(tracker->untrackingServices);
     free(tracker);
 	return CELIX_SUCCESS;
 }
@@ -223,7 +222,7 @@ celix_status_t serviceTracker_close(service_tracker_t* tracker) {
             if (nrOfTrackedEntries > 0) {
                 tracked = celix_arrayList_get(tracker->trackedServices, 0);
                 celix_arrayList_removeAt(tracker->trackedServices, 0);
-                celix_arrayList_add(tracker->untrackingServices, tracked);
+                tracker->untrackedServiceCount++;
             }
             celixThreadMutex_unlock(&tracker->mutex);
 
@@ -231,7 +230,7 @@ celix_status_t serviceTracker_close(service_tracker_t* tracker) {
                 int currentSize = nrOfTrackedEntries - 1;
                 serviceTracker_untrackTracked(tracker, tracked, currentSize, currentSize == 0);
                 celixThreadMutex_lock(&tracker->mutex);
-                celix_arrayList_remove(tracker->untrackingServices, tracked);
+                tracker->untrackedServiceCount--;
                 celixThreadCondition_broadcast(&tracker->condUntracking);
                 celixThreadMutex_unlock(&tracker->mutex);
             }
@@ -528,7 +527,7 @@ static celix_status_t serviceTracker_untrack(service_tracker_t* tracker, service
             remove = tracked;
             //remove from trackedServices to prevent getting this service, but don't destroy yet, can be in use
             celix_arrayList_removeAt(tracker->trackedServices, i);
-            celix_arrayList_add(tracker->untrackingServices, remove);
+            tracker->untrackedServiceCount++;
             break;
         }
     }
@@ -539,12 +538,13 @@ static celix_status_t serviceTracker_untrack(service_tracker_t* tracker, service
     if (remove != NULL) {
         serviceTracker_untrackTracked(tracker, remove, size, true);
         celixThreadMutex_lock(&tracker->mutex);
-        celix_arrayList_remove(tracker->untrackingServices, remove);
+        tracker->untrackedServiceCount--;
+        celixThreadCondition_broadcast(&tracker->condUntracking);
         celixThreadMutex_unlock(&tracker->mutex);
     } else {
         //ensure no untrack is still happening (to ensure it safe to unregister service)
         celixThreadMutex_lock(&tracker->mutex);
-        while (celix_arrayList_size(tracker->untrackingServices) > 0) {
+        while (tracker->untrackedServiceCount > 0) {
             celixThreadCondition_wait(&tracker->condUntracking, &tracker->mutex);
         }
         celixThreadMutex_unlock(&tracker->mutex);
@@ -593,7 +593,7 @@ static celix_status_t serviceTracker_invokeRemovingService(service_tracker_t *tr
     if (tracker->remove != NULL) {
         tracker->remove(handle, tracked->service);
     }
-    if (tracker->addWithProperties != NULL) {
+    if (tracker->removeWithProperties != NULL) {
         tracker->removeWithProperties(handle, tracked->service, tracked->properties);
     }
     if (tracker->removeWithOwner != NULL) {
@@ -632,28 +632,35 @@ celix_service_tracker_t* celix_serviceTracker_create(
     return celix_serviceTracker_createWithOptions(ctx, &opts);
 }
 
-celix_service_tracker_t* celix_serviceTracker_createWithOptions(
-        bundle_context_t *ctx,
-        const celix_service_tracking_options_t *opts
-) {
-    celix_service_tracker_t *tracker = NULL;
+celix_service_tracker_t* celix_serviceTracker_createClosedWithOptions(celix_bundle_context_t* ctx,
+                                                                      const celix_service_tracking_options_t* opts) {
+    celix_service_tracker_t* tracker = NULL;
     const char* serviceName = NULL;
-    char *filter = NULL;
+    char* filter = NULL;
     assert(ctx != NULL);
     assert(opts != NULL);
     serviceName = opts->filter.serviceName == NULL ? "*" : opts->filter.serviceName;
-    filter = celix_serviceRegistry_createFilterFor(ctx->framework->registry, opts->filter.serviceName, opts->filter.versionRange, opts->filter.filter);
-    if(filter == NULL) {
-        celix_framework_log(ctx->framework->logger, CELIX_LOG_LEVEL_ERROR, __FUNCTION__, __BASE_FILE__, __LINE__,
+    filter = celix_serviceRegistry_createFilterFor(
+        ctx->framework->registry, opts->filter.serviceName, opts->filter.versionRange, opts->filter.filter);
+    if (filter == NULL) {
+        celix_framework_log(ctx->framework->logger,
+                            CELIX_LOG_LEVEL_ERROR,
+                            __FUNCTION__,
+                            __BASE_FILE__,
+                            __LINE__,
                             "Error cannot create filter.");
         return NULL;
     }
     tracker = calloc(1, sizeof(*tracker));
-    if(tracker == NULL) {
+    if (tracker == NULL) {
         free(filter);
-        celix_framework_log(ctx->framework->logger, CELIX_LOG_LEVEL_ERROR, __FUNCTION__, __BASE_FILE__, __LINE__,
+        celix_framework_log(ctx->framework->logger,
+                            CELIX_LOG_LEVEL_ERROR,
+                            __FUNCTION__,
+                            __BASE_FILE__,
+                            __LINE__,
                             "No memory for tracker.");
-       return NULL;
+        return NULL;
     }
 
     tracker->context = ctx;
@@ -661,7 +668,7 @@ celix_service_tracker_t* celix_serviceTracker_createWithOptions(
     tracker->filter = filter;
     tracker->state = CELIX_SERVICE_TRACKER_CLOSED;
 
-    //setting callbacks
+    // setting callbacks
     tracker->callbackHandle = opts->callbackHandle;
     tracker->set = opts->set;
     tracker->add = opts->add;
@@ -680,12 +687,22 @@ celix_service_tracker_t* celix_serviceTracker_createWithOptions(
     celixThreadCondition_init(&tracker->condTracked, NULL);
     celixThreadCondition_init(&tracker->condUntracking, NULL);
     tracker->trackedServices = celix_arrayList_create();
-    tracker->untrackingServices = celix_arrayList_create();
+    tracker->untrackedServiceCount = 0;
     tracker->currentHighestServiceId = -1;
 
     tracker->listener.handle = tracker;
-    tracker->listener.serviceChanged = (void *) serviceTracker_serviceChanged;
+    tracker->listener.serviceChanged = (void*)serviceTracker_serviceChanged;
+    return tracker;
+}
 
+celix_service_tracker_t* celix_serviceTracker_createWithOptions(
+        bundle_context_t *ctx,
+        const celix_service_tracking_options_t *opts
+) {
+    celix_service_tracker_t *tracker = celix_serviceTracker_createClosedWithOptions(ctx, opts);
+    if(tracker == NULL) {
+       return NULL;
+    }
     serviceTracker_open(tracker);
     return tracker;
 }
@@ -697,6 +714,29 @@ void celix_serviceTracker_destroy(celix_service_tracker_t *tracker) {
     }
 }
 
+static celix_tracked_entry_t* celix_serviceTracker_findHighestRankingService(service_tracker_t *tracker, const char* serviceName) {
+    //precondition tracker->mutex locked
+    celix_tracked_entry_t* highest = NULL;
+    for (int i = 0; i < celix_arrayList_size(tracker->trackedServices); ++i) {
+        celix_tracked_entry_t* tracked = (celix_tracked_entry_t *) arrayList_get(tracker->trackedServices, i);
+        if (serviceName == NULL || (serviceName != NULL && tracked->serviceName != NULL &&
+                                    celix_utils_stringEquals(tracked->serviceName, serviceName))) {
+            if (highest == NULL) {
+                highest = tracked;
+            } else {
+                int compare = celix_utils_compareServiceIdsAndRanking(
+                        tracked->serviceId, tracked->serviceRanking,
+                        highest->serviceId, highest->serviceRanking
+                );
+                if (compare < 0) {
+                    highest = tracked;
+                }
+            }
+        }
+    }
+    return highest;
+}
+
 bool celix_serviceTracker_useHighestRankingService(service_tracker_t *tracker,
                                                    const char *serviceName /*sanity*/,
                                                    double waitTimeoutInSeconds /*0 -> do not wait */,
@@ -704,59 +744,27 @@ bool celix_serviceTracker_useHighestRankingService(service_tracker_t *tracker,
                                                    void (*use)(void *handle, void *svc),
                                                    void (*useWithProperties)(void *handle, void *svc, const celix_properties_t *props),
                                                    void (*useWithOwner)(void *handle, void *svc, const celix_properties_t *props, const celix_bundle_t *owner)) {
-    bool called = false;
-    celix_tracked_entry_t *tracked = NULL;
-    celix_tracked_entry_t *highest = NULL;
-    unsigned int i;
-    struct timespec begin = celix_gettime(CLOCK_MONOTONIC);
-    double remaining = waitTimeoutInSeconds > INT_MAX ? INT_MAX : waitTimeoutInSeconds;
-    remaining = remaining < 0 ? 0 : remaining;
-    double elapsed = 0;
-    long seconds = remaining;
-    long nanoseconds = (remaining - seconds) * CELIX_NS_IN_SEC;
-
-    //first lock tracker and get highest tracked entry
+    //first lock tracker and get highest ranking tracked entry
     celixThreadMutex_lock(&tracker->mutex);
-    while (highest == NULL) {
-        unsigned int size = arrayList_size(tracker->trackedServices);
-
-        for (i = 0; i < size; i++) {
-            tracked = (celix_tracked_entry_t *) arrayList_get(tracker->trackedServices, i);
-            if (serviceName == NULL || (serviceName != NULL && tracked->serviceName != NULL &&
-                                        celix_utils_stringEquals(tracked->serviceName, serviceName))) {
-                if (highest == NULL) {
-                    highest = tracked;
-                } else {
-                    int compare = celix_utils_compareServiceIdsAndRanking(
-                            tracked->serviceId, tracked->serviceRanking,
-                            highest->serviceId, highest->serviceRanking
-                    );
-                    if (compare < 0) {
-                        highest = tracked;
-                    }
-                }
-            }
-        }
-        if(highest == NULL && (seconds > 0 || nanoseconds > 0)) {
-            celixThreadCondition_timedwaitRelative(&tracker->condTracked, &tracker->mutex, seconds, nanoseconds);
-            elapsed  = celix_elapsedtime(CLOCK_MONOTONIC, begin);
-            remaining = remaining > elapsed ? (remaining - elapsed) : 0;
-            seconds = remaining;
-            nanoseconds = (remaining - seconds) * CELIX_NS_IN_SEC;
-        } else {
-            // highest found or timeout
+    struct timespec absTime = celixThreadCondition_getDelayedTime(waitTimeoutInSeconds);
+    celix_tracked_entry_t* highest = celix_serviceTracker_findHighestRankingService(tracker, serviceName);
+    while (highest == NULL && waitTimeoutInSeconds > 0) {
+        celix_status_t waitStatus = celixThreadCondition_waitUntil(&tracker->condTracked, &tracker->mutex, &absTime);
+        if (waitStatus == ETIMEDOUT) {
             break;
         }
+        highest = celix_serviceTracker_findHighestRankingService(tracker, serviceName);
     }
-    if (highest != NULL) {
-        //highest found lock tracked entry and increase use count
+    if (highest) {
+        // highest found, increase use count
         tracked_retain(highest);
     }
-    //unlock tracker so that the tracked entry can be removed from the trackedServices list if unregistered.
+    // unlock tracker so that the tracked entry can be removed from the trackedServices list if unregistered.
     celixThreadMutex_unlock(&tracker->mutex);
 
-    if (highest != NULL) {
-        //got service, call, decrease use count an signal useCond after.
+    bool called = false;
+    if (highest) {
+        //got service, call, decrease use count a signal useCond after.
         if (use != NULL) {
             use(callbackHandle, highest->service);
         }
