@@ -23,6 +23,7 @@
 #include "celix_string_hash_map.h"
 #include "celix_long_hash_map.h"
 #include <random>
+#include <atomic>
 
 class HashMapTestSuite : public ::testing::Test {
 public:
@@ -201,7 +202,7 @@ TEST_F(HashMapTestSuite, DestroyHashMapWithSimpleRemovedCallback) {
     celix_stringHashMap_destroy(sMap);
 
     celix_long_hash_map_create_options_t lOpts{};
-    lOpts.simpledRemoveCallback = free;
+    lOpts.simpleRemovedCallback = free;
     auto* lMap = celix_longHashMap_createWithOptions(&lOpts);
     EXPECT_TRUE(lMap != nullptr);
     EXPECT_EQ(0, celix_longHashMap_size(lMap));
@@ -545,3 +546,13 @@ TEST_F(HashMapTestSuite, IterateWithRemoveTest) {
     EXPECT_TRUE(celix_longHashMapIterator_isEnd(&iter2));
     celix_longHashMap_destroy(lMap);
 }
+
+TEST_F(HashMapTestSuite, StringHashMapCleanup) {
+    celix_autoptr(celix_string_hash_map_t) map = celix_stringHashMap_create();
+    EXPECT_NE(nullptr, map);
+}
+
+TEST_F(HashMapTestSuite, LongHashMapCleanup) {
+    celix_autoptr(celix_long_hash_map_t) map = celix_longHashMap_create();
+    EXPECT_NE(nullptr, map);
+}
